#! /usr/bin/perl
#
# $Id$
#

package main;
use lib "./lib";
BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 153 $ =~ /\d+/g); $r[0]};
    use ParamParser;
    &ParamParser::SetDefaultBehaviour('exit_on_getopt_error');
    &ParamParser::SetDefaultUsage(sub {print "clism_install.pl\n";});
    # clism est le nom d'administrateur
    # password est le mot de passe correspondant    
    $main::O_PARAM = New ParamParser('GETOPTLONG',("help"));

    #$main::O_PARAM = New ParamParser();
    $main::O_CFG = New ParamParser('../cfg/clism.cfg');
}

use strict;
use Carp;
#use clism::DBMS;
#use clism::utilisateur;
use File::Basename;
use Cwd;
use General;

MAIN:
{
    # Installation de clism
    print "ATTENTION - VOUS DEVEZ AVOIR L'AUTORISATION SUDO POUR INSTALLER CLISM EN UTILISANT CE SCRIPT\n";

    my $continue_flg = &Ask("On y va ? ([Y/N])",'N',\&YesOrNo);
    if (uc($continue_flg) ne 'Y')
    {
        print "He ben non. Bye\n";
        exit;
    }
    
    # Ask the user a lot of data
    my $project_name        = &Ask("Nom du projet                         ",'clism',\&OnlyLN);
    if ($project_name eq 'distrib')
    {
        print "Desole, distrib ne peut pas etre un nom de projet\n";
        exit;
    }
    if ($project_name ne 'clism')
    {
        $project_name = "clism_" . $project_name unless ($project_name =~ /^clism_/);
    }
    
    my $project_description = &Ask("Quelques mots de description    (50 chars max) ",'',\&Only50chars);
    my $mail                = &Ask("Le mail de l'administrateur                    ",'',\&NoBlankEmail);
    my $prefix              = &Ask("Le Prefixe (3 caracteres, une souche aura pour id GMK1000)",'GMK',\&Only3chars);
    my $web_server_name     = &Ask("Le nom du serveur web (ex: truc.machin.fr)          ",'localhost',\&NoStartHttp);
    my $directory           = cwd;
    
    #$directory           = &Ask("Le repertoire d'installation ",$directory,\&NoBlankAbsDirExists);
    my $url                 = &Ask("L'URL souhaitee (pour http://$web_server_name/clism => entrer uniquement clism) ",'clism');
    
    my $postgres_host       = &Ask("Le nom du serveur postgres (si rien, on utilise les sockets unix) ",'');
    my $postgres_user       = &Ask("Le login de l'administateur postgres                  ",'postgres');
    
    my $default_guest       = &Ask("Voulez-vous donner l'acces en lecture sans authentification ? (o/N)",'N');
    my $default_pass = '';
    
    # sql file generation
    &CreateSql($project_name,$project_description,$prefix,$mail);
    
	# utilisateurs postgresql
	&CreatePgRoles($project_name,$postgres_host,$postgres_user);
	
    # import to postgresql
    &CreateSql($project_name,$project_description,$prefix,$mail);
    &CreateDb($project_name,$postgres_host,$postgres_user);
    
    # cfg file generation
    &CreateCfg($project_name,$directory,$web_server_name,$url,$postgres_host,$default_guest,$project_description,$mail);
    
    # Tell the admin to modify the pg_hba.conf
    &Modifpg_hba($project_name,$postgres_host,$web_server_name);
    
    # Tell the admin to modify the Apache configuration
    &ModifApache($directory,$web_server_name,$url);

    # Restart Apache et Postgresql
    &RestartApachePostgresql();
    
}

=head2 procedure Modifpg_hba

 Title     : Modifpg_hba
 Procedure : Explique la ligne a ajouter a pg_hba.conf
             si le nom de serveur postgres est '', la ligne est de type 'local'
             sinon, elle est de type host
 Args      : nom du projet, nom du serveur postgres et du serveur web 
 
=cut
sub Modifpg_hba
{
    my ($name,$postgres_host,$web_host) = @_;

    print "\n-----------------------------------------------------------", "\n";
    print "Vous pouvez copier-coller la ligne suivante dans le fichier pg_hba.conf:\n\n";
    if ($postgres_host ne '')
    {    
        my ($n,$a,$t,$l,@d)  = gethostbyname($web_host);
        my ($a1,$a2,$a3,$a4) = unpack('W4',$d[0]);
        my $ip = "$a1.$a2.$a3.$a4";
        print "host    $name       $name,+${name}_all $ip/32 md5\n";
    }
    else
    {
        print "local   $name  $name,+${name}_all md5\n";
    }
}  

=head2 procedure ModifApache

 Title     : ModifApache
 Procedure : Explique la modif a faire dans la config apache
 Args      : nom du projet, nom du serveur postgres et du serveur web 
 
=cut
sub ModifApache
{
    my ($directory,$web_server_name,$url) = @_;

    print "\n-----------------------------------------------------------", "\n";
    print "Si ce n'est deja fait, vous pouvez copier-coller les lignes suivantes dans la config Apache:\n";
    print "(pensez a modifier la ligne Allow from\n\n";
    
    print "ScriptAlias \"/${url}/cgi-bin\" \"${directory}/cgi-bin\" \n";
    print "Alias \"/$url\" \"${directory}/web\" \n";
    print "<Location \"/$url\">\n";
    print "   Order deny,allow\n";
    print "   Deny from all\n";
    print "   Allow from chez-vous.fr\n";
    print "</Location>\n";
    print "Redirect permanent \"/${url}/index.html\" \"http://${web_server_name}/${url}/cgi-bin/index.cgi\"\n";
}  

=head2 procedure RestartApachePostgresql

 Title : RestartApachePostgresql
 
=cut
sub RestartApachePostgresql
{
    print "\n-----------------------------------------------------------", "\n";
    print "REDEMARREZ Apache\n";
    print "REDEMARREZ Postgresql\n";
}    

=head2 procedure CreateCfg

 Title : CreateCfg
 Usage : &CreateCfg($project_name,$directory,$web_server_name,$url,$postgres_host,$default_guest);
 Procedure : Cree le fichiers de configuration 
 Args: tout plein

=cut
sub CreateCfg
{
    my ($name,$directory,$web_server_name,$url,$postgres_host,$default_guest,$desc,$mail) = @_;
    
    my $file_name = ($name eq 'clism') ? 'clism_clism.cfg' : "$name.cfg";
    
    my $default_pass = "";
    if ($default_guest =~ /o/i)
    {
        $default_pass = &Ask("Entrez a nouveau le mot de passe invite ",'',\&NoBlank);
    }
    
    $url =~ s!/$!!;
    $url .= '/';
	my $fh_clism = new IO::File('cfg/clism_distrib.cfg') or die 'peux pas ouvrir cfg/clism_distrib.cfg';
	my $fh_prj   = new IO::File("cfg/$file_name",'w') or die "peux pas ouvrir cfg/$file_name";
	while (my $line = <$fh_clism>)
	{
		chomp $line;
		$line =~ s!^SERVERNAME=.*!SERVERNAME=http://$web_server_name!;
		$line =~ s/^ROOT=.*/ROOT=$directory/;
		$line =~ s/^HTTP_ROOT=.*/HTTP_ROOT=$url/;
        $line =~ s/^CLISM_DESC=.*/CLISM_DESC=$desc/;
        $line =~ s/^CLISM_MAIL=.*/CLISM_MAIL=$mail/;
		$line =~ s/^db_host=.*/db_host=$postgres_host/;
		$line =~ s/^db_name=.*/db_name=$name/;
        $line =~ s/^db_login_admin=.*/db_login_admin=$name/;
        $line =~ s/^db_login_default=.*/db_login_default=${name}_guest/;
        if ($default_guest =~ /o/i)
        {
            $line =~ s/^db_pass_default=.*/db_pass_default=$default_pass/;
        }
        else
        {
            $line =~ s/^db_pass_default=.*/#db_pass_default=/;
        }
		print $fh_prj "$line\n";
	}
}

=head2 procedure CreatePgRole

 Title : CreatePgRole
 Usage : &CreatePgRole($name,$pghost,$pguser);
 Procedure : Cree les roles
 Args: nom du projet, hote et utilisateur postgres

=cut

sub CreatePgRoles	
{
	my ($name,$pghost,$pguser) = @_;
    $pghost                    = "-h $pghost " if ($pghost ne "");

    print "\n-----------------------------------------------------------", "\n";
	print "Utilisateur $name         ==> l'administrateur clism\n";
	`sudo -u $pguser createuser $pghost -S -D -R --login --pwprompt $name`;
	
    print "\n-----------------------------------------------------------", "\n";
	print "Utilisateur ${name}_guest ==> lecture de la BD \n";
	`sudo -u $pguser createuser $pghost -S -D -R --login --pwprompt ${name}_guest`;
	
    print "\n-----------------------------------------------------------", "\n";
	print "Groupe ${name}_all ==> pour tous les utilisateurs clism_* \n";
	`sudo -u $pguser createuser $pghost --no-login -S -D -R ${name}_all`;
	`sudo -u $pguser psql -c 'ALTER GROUP ${name}_all ADD USER ${name}_guest'`;
}
	
=head2 procedure CreateSql

 Title : CreateSql
 Usage : &Createsql($name,$description,$prefix,$mail)
 Procedure : Lit le fichier clism_distrib.sql et le modifie avant de le sauver
 Args: nom et description du projet, prefix, adresse mail de l'admin

=cut

sub CreateSql
{
	my ($name,$desc,$prefix,$mail) = @_;
	
    print "\n-----------------------------------------------------------", "\n";
	print "Generation du fichier sql/$name.sql qui serivra a remplir la base\n";
    
	my $fh_clism = new IO::File('sql/clism_distrib.sql') or die 'peux pas ouvrir sql/clism_distrib.sql';
	my $fh_prj   = new IO::File("sql/$name.sql",'w') or die "peux pas ouvrir sql/$name.sql";
	while (my $line = <$fh_clism>)
	{
		chomp $line;
		$line =~ s/CLISM/$name/g;
		$line =~ s/ADMINDESC/$desc/;
		$line =~ s/ADMINMAIL/$mail/;
		$line =~ s/ADMIN/$name/;
		$line =~ s/PRE/$prefix/;
		print $fh_prj "$line\n";
	}
}

=head2 procedure CreateDb

 Title : CreateDb
 Usage : &CreateDb($name,$pghost,$pguser);
 Procedure : Cree et importe la base de donnees depuis le fichier cree par CreateSql
 Args: nom du projet, hote et utilisateur postgres

=cut
sub CreateDb
{
	my ($name,$pghost,$pguser) = @_;
    $pghost                    = "-h $pghost " if ($pghost ne "");

    print "\n-----------------------------------------------------------", "\n";
	print "Creation de la base $name\n";
	`sudo -u $pguser createdb $pghost $name`;

    print "\n-----------------------------------------------------------", "\n";    
    print "Remplissage de la base $name\n";
    `sudo -u $pguser psql $pghost -f sql/$name.sql $name >/dev/null 2>&1`;
}
	
=head2 function Ask

 Title        : Ask
 Usage        : my $variable = &Ask("Enter something ",'DEFAULT',\&Check);
 Fonction     : Print a message to stdout and read the answer from stdin
                If '' is read, the default value is taken
                If specified, the function \&Check is called, and if this function returns FALSE
                the message is printed again, and stdin is read again
 Returns      : The value entered by the user, validated
 Args         : $msg : The message to print
                $default(optional): The default value
                \&Check (optional): The function called for validation

=cut
sub Ask
{
    my ($msg,$default,$test) = @_;
    $default = "" unless defined $default;
    
    my $answer = "";
    do
    {
        print $msg;
        if ($default ne '')
        {
            print " ($default) ";
        }
        $answer = <STDIN>;
        chomp($answer);
        if ( $answer eq "" )
        {
            $answer = $default;
        }
    }
    while (defined $test and &$test($answer) == &FALSE);
    return $answer;
}

# YesOrNo = must be Y or N
sub YesOrNo
{
    return ($_[0] =~ /^[YN]$/i);
}


# OnlyLN = must be letters of numbers or _
sub OnlyLN
{
    return ($_[0] =~ /^[a-z0-9_]+$/i);
}
sub Only50chars
{
    return ($_[0] =~ /^[a-z0-9 .\-_]{1,50}$/i);
}
sub Only3chars
{
    return ($_[0] =~ /^[A-Z]{3}$/);
}
sub NoStartHttp
{
    return ($_[0] !~ /^http:\/\//);
}

# NoBlank = must be ne ""
sub NoBlank
{
    return ($_[0] !~ /^\s*$/) ? &TRUE : &FALSE;
}

# NoBlankAbsDirExists = not blank, must be an absolute path towards an existing directory
sub NoBlankAbsDirExists
{
    return ($_[0] =~ /^\// && (-d $_[0] || -l $_[0])) ? &TRUE : &FALSE;
}


# NoBlankEmail = must contain an @ character
sub NoBlankEmail
{
    return ($_[0] =~ /^.+@.+/) ? &TRUE : &FALSE;
}


