#! /usr/bin/perl
#
# $Id$
#

package main;
use lib "./lib";
BEGIN
{
	
    our $VERSION = do {my @r = (q$Rev: 153 $ =~ /\d+/g); $r[0]};
    use ParamParser;
    &ParamParser::SetDefaultBehaviour('exit_on_getopt_error');
    &ParamParser::SetDefaultUsage(sub {print "clism_install.pl\n";});
    # clism est le nom d'administrateur
    # password est le mot de passe correspondant    
    $main::O_PARAM = New ParamParser('GETOPTLONG',("help"));

    #$main::O_PARAM = New ParamParser();
    $main::O_CFG = New ParamParser('../cfg/clism.cfg');
}

use strict;
use Carp;
#use clism::DBMS;
#use clism::utilisateur;
use File::Basename;
use Cwd;
use General;

MAIN:
{
    # Installation de clism
    print "WARNING - TO USE THIS SCRIPT YOU SHOULD BE ABLE TO USE SUDO - PLEASE ASK YOUR SYSTEM ADMINISTRATOR IF NECCESARY\n";

    my $continue_flg = &Ask("Continue ? ([Y/N])",'N',\&YesOrNo);
    if (uc($continue_flg) ne 'Y')
    {
        print "Interrupted by the user. Bye\n";
        exit;
    }
    
    # Ask the user a lot of data
    my $project_name        = &Ask("What is the project name ?                         ",'clism',\&OnlyLN);
    my $postgres_host       = &Ask("What is the postgresql host name (if blank use unix socket)",'');
    my $postgres_user       = &Ask("Enter the postgresql admin user ",'postgres');
    if ($project_name eq 'distrib')
    {
        print "Desole, distrib ne peut pas etre un nom de projet\n";
        exit;
    }
    if ($project_name ne 'clism')
    {
        $project_name = "clism_" . $project_name unless ($project_name =~ /^clism_/);
    }
    
    # bd postgresql
    &DropDb($project_name,$postgres_host,$postgres_user);

	# utilisateurs postgresql
	&DropPgRoles($project_name,$postgres_host,$postgres_user);
    
    # cfg file removal
    &RemoveCfg($project_name);
    
    # Tell the admin to modify the pg_hba.con
    &Modifpg_hba($project_name);
    
    # Restart Apache
    &RestartApachePostgresql();
}

=head2 procedure Modifpg_hba

 Title : Modifpg_hba
 
=cut
sub Modifpg_hba
{
    my ($name) = @_;
    
    print "\n-----------------------------------------------------------", "\n";
    print "Vous pouvez supprimer la ligne $name dans le fichier pg_hba.conf:\n";
    print "Pour la trouver tapez:\n";
    print "grep $name .../pg_hba.conf\n";
}  
=head2 procedure RemoveCfg

 Title : RemoveCfg
 Usage : &RemoveCfg($project_name);
 Procedure : Supprime le fichiers de configuration 
 Args: nom du projet

=cut
sub RemoveCfg
{
    my ($name) = @_;

    $name = "clism_clism" if ($name eq 'clism');

    print "\n-----------------------------------------------------------", "\n";
    print "Suppression du fichier cfg/$name.cfg\n";
    unlink "cfg/$name.cfg";
}

=head2 procedure RestartApachePostgresql

 Title : RestartApachePostgresql
 
=cut
sub RestartApachePostgresql
{
    print "\n-----------------------------------------------------------", "\n";
    print "REDEMARREZ Apache\n";
    print "REDEMARREZ Postgresql\n";
}    

=head2 procedure DropPgRoles

 Title : DropPgRoles
 Usage : &DropPgRoles($name,$pghost,$pguser);
 Procedure : Supprime les roles
 Args: nom du projet, hote et utilisateur postgres

=cut

sub DropPgRoles	
{
	my ($name,$pghost,$pguser) = @_;
    $pghost                    = "-h $pghost " if ($pghost ne "");

    print "\n-----------------------------------------------------------", "\n";
	print "Suppression de l'utilisateur $name\n";
	`sudo -u $pguser dropuser $pghost $name`;
	
    print "\n-----------------------------------------------------------", "\n";
	print "Suppression de l'utilisateur ${name}_guest\n";
	`sudo -u $pguser dropuser $pghost ${name}_guest`;
	
    print "\n-----------------------------------------------------------", "\n";
	print "Suppression du groupe ${name}_all\n";
	`sudo -u $pguser dropuser $pghost ${name}_all`;
}
	
=head2 procedure DropDb

 Title : DropDb
 Usage : &DropDb($name,$pghost,$pguser);
 Procedure : Supprime la base de donnees
 Args: nom du projet, hote et utilisateur postgres

=cut
sub DropDb
{
	my ($name,$pghost,$pguser) = @_;
    $pghost                    = "-h $pghost " if ($pghost ne "");

	print "Suppression de la base $name\n";
	`sudo -u $pguser dropdb $pghost $name`;
}
	
=head2 function Ask

 Title        : Ask
 Usage        : my $variable = &Ask("Enter something ",'DEFAULT',\&Check);
 Fonction     : Print a message to stdout and read the answer from stdin
                If '' is read, the default value is taken
                If specified, the function \&Check is called, and if this function returns FALSE
                the message is printed again, and stdin is read again
 Returns      : The value entered by the user, validated
 Args         : $msg : The message to print
                $default(optional): The default value
                \&Check (optional): The function called for validation

=cut
sub Ask
{
    my ($msg,$default,$test) = @_;
    $default = "" unless defined $default;
    
    my $answer = "";
    do
    {
        print $msg;
        if ($default ne '')
        {
            print " ($default) ";
        }
        $answer = <STDIN>;
        chomp($answer);
        if ( $answer eq "" )
        {
            $answer = $default;
        }
    }
    while (defined $test and &$test($answer) == &FALSE);
    return $answer;
}

# YesOrNo = must be Y or N
sub YesOrNo
{
    return ($_[0] =~ /^[YN]$/i);
}


# OnlyLN = must be letters of numbers or _
sub OnlyLN
{
    return ($_[0] =~ /^[a-z0-9_]+$/i);
}
sub Only50chars
{
    return ($_[0] =~ /^[a-z0-9 .\-_]{1,50}$/i);
}
sub Only3chars
{
    return ($_[0] =~ /^[A-Z]{3}$/);
}
sub StartHttp
{
    return ($_[0] =~ /^http:\/\/.+\..+$/);
}

# NoBlank = must be ne ""
sub NoBlank
{
    return ($_[0] !~ /^\s*$/) ? &TRUE : &FALSE;
}

# NoBlankAbsDirExists = not blank, must be an absolute path towards an existing directory
sub NoBlankAbsDirExists
{
    return ($_[0] =~ /^\// && -d $_[0]) ? &TRUE : &FALSE;
}


# NoBlankEmail = must contain an @ character
sub NoBlankEmail
{
    return ($_[0] =~ /^.+@.+/) ? &TRUE : &FALSE;
}


