#!/usr/bin/perl -I../lib

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# sebastien.carrere@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for managing strains and plasmids in the LIPM

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod
=head1 NAME

	index_en.cgi -

=head1 SYNOPSIS

	this script manages the web interface when using english language

=head1 DESCRIPTION

	- should be called by the web server,
=cut

use strict;

our $O_CGI;
our $O_PARAM;
our $O_CFG;
our $O_WB;
our $O_LOGGER;

BEGIN
{
    # Finding the libraries
    use File::Basename;
    my $dirprg=dirname($0);
    unshift(@INC, "$dirprg/../lib");

    use CGI qw(:standard);
    use CGI::Carp qw/fatalsToBrowser/;
    use ParamParser;
    use WebBuilder;
    use Logger;
    
    # Initializing $O_CGI
    $O_CGI = new CGI;

    # Initializing $O_PARAM and $O_CFG
    $O_PARAM = New ParamParser('INIT');
    #$O_PARAM->SetAuthorizedCharacters('[>/0-9a-zA-Z:_.@+\#\-\s=,\?\(\)]');
    $O_PARAM->UnsetBehaviour('assert_value_secure');
    $O_PARAM->Update('CGIPM','A');

    # Si logout, supprimer l'utilisateur et le mot de passe et passer a la fonction par defaut
    # Supprimer aussi la session afin de forcer a la creation d'une nouvelle session
    if ($O_PARAM->Get('__wb_function') eq 'Logout')
    {
        $O_PARAM->Delete('__wb_login');
        $O_PARAM->Set('__wb_function',     'MainScreen');
        $O_PARAM->Delete('__wb_session');
    }

    # Fonction par defaut
    $O_PARAM->SetUnlessDefined('__wb_function',     'MainScreen');
    #$O_PARAM->SetUnlessDefined('__wb_main_menu',    $O_PARAM->Get('__wb_function'));

    # Les logins sont obligatoirement clism OU clism_xxxx
    if ($O_PARAM->IsDefined('__wb_login'))
    {
        my $login = $O_PARAM->Get('__wb_login');
        if ($login !~ /^clism/)
        {
            $login = 'clism_' . $login;
            $O_PARAM->Set('__wb_login',$login);
        }
    }
    
    # Initializing the configuration
    # on cherche la config dont on a passe le nom
    my $clism_cfg = '';
    if ($O_PARAM->IsDefined('__wb_nom'))
    {
        my $nom = $O_PARAM->Get('__wb_nom');
        $clism_cfg= "../cfg/clism_$nom.cfg";
    }
    
    # On trouve le fichier de conf: tant mieux, on peut initialiser $O_CFG
    if ( ($clism_cfg ne '') and (-r $clism_cfg))
    {
        $O_CFG = New ParamParser($clism_cfg);
        $O_CFG->SetUnlessDefined('db_login_admin','clism');

        $O_CFG->SetUnlessDefined('default_lang','fr');
        $O_PARAM->SetUnlessDefined('__wb_lang',$O_CFG->Get('default_lang'));
        
        $O_PARAM->UnsetBehaviour('assert_strict');
        
        my $new_session = !$O_PARAM->IsDefined('__wb_session');
        
        # Initialize $O_WB
        my $desc_file = $O_CFG->Get('ROOT').'/'.$O_CFG->Get('XML_DIR').'/'.$O_PARAM->Get('__wb_lang').'/'.$O_CFG->Get('LAYOUT');
        
        # substitutions - dependent de l'utilisateur
        my ($s_login,$s_logout,$s_update,$s_admin,$s_strain,$s_plasmid,$s_biblio)=('item01','item01','item02','item03','item10','item20','item25');
        $s_login   = "" if ($O_PARAM->IsDefined('__wb_login'));
        $s_logout  = "" if (!$O_PARAM->IsDefined('__wb_login'));
        $s_update  = "" unless ($O_PARAM->IsDefined('__wb_login') and $O_PARAM->Get('__wb_login') ne $O_CFG->Get('db_login_default'));
        $s_admin   = "" unless ($O_PARAM->IsDefined('__wb_login') and $O_PARAM->Get('__wb_login') eq $O_CFG->Get('db_login_admin'));
        #warn $O_CFG->Get('db_login_default');
        if (!($O_CFG->IsDefined('db_login_default') and $O_CFG->IsDefined('db_pass_default')) and !$O_PARAM->IsDefined('__wb_login'))
        {
            $s_strain  = "";
            $s_plasmid = "";
            $s_biblio  = "";
        }
        
        my $s_lang = $O_PARAM->Get('__wb_lang');
        my $tmp_dir= $O_CFG->Get('ROOT') . '/' . $O_CFG->Get('TMP_DIR');
        
        $O_WB = New WebBuilder($O_PARAM,"",$desc_file,
            '%I' => $s_login,
            '%O' => $s_logout,
            '%U' => $s_update,
            '%A' => $s_admin,
            '%S' => $s_strain,
            '%P' => $s_plasmid,
            '%B' => $s_biblio,
            '%L' => $s_lang,
            '%g' => $ENV{SCRIPT_NAME},
            '%T' => $tmp_dir);
        
        $O_WB->UnsetBehaviour('encoded_url');
        $O_WB->SetBehaviour('send_parameters');
        $O_WB->SetBehaviour('using_yui');
        
        # Init the session directory and redirect stderr to clism.err
        $O_WB->SetBehaviour('use_session');
        $O_WB->SetBehaviour('session_autoclean');

        $O_WB->InitSession();
        close(STDERR);
        open(STDERR, ">> clism.err") or die "Cannot open clism.err for writing";

        # Init $o_logger et le garder dans $o_param (PROVISOIRE !)
        # Init $O_LOGGER 
        if ($new_session==1)
        {
            $O_LOGGER = New Logger('>>clism.log',$O_CFG->Get('LOG_LEVEL'),'NOCOMPRESSION');
            $O_LOGGER->Trace("NOUVELLE SESSION - __wb_function = ".$O_PARAM->Get("__wb_function")."\n",'.','1');
        }
        else
        {
            $O_LOGGER = New Logger('>>clism.log',$O_CFG->Get('LOG_LEVEL'),'NOCOMPRESSION NEWQUIET');
            if (!$O_PARAM->IsDefined('__wba_ajax') or $O_PARAM->Get('__wba_ajax') eq '0')
            {
                $O_LOGGER->Trace("NOUVELLE REQUETE - __wb_function = ".$O_PARAM->Get("__wb_function")."\n",'*','1');
            }
            else
            {
                $O_LOGGER->Trace("NOUVELLE REQUETE AJAX (".$O_PARAM->Get("__wba_ajax").") - __wb_autocomplete_field = ".$O_PARAM->Get("__wb_autocomplete_field")."\n",'*','1');
            }
        }
        
        # TODO - NE SERT A RIEN !
        #$O_PARAM->Set('O_LOGGER',$O_LOGGER);
        
        use Data::Dumper;
        $O_LOGGER->Trace("O_PARAM = ".Dumper($O_PARAM->{__h_opt}),'','4');
    }
    
    # On ne le trouve pas: tant pis on affiche une liste de configurations
    else
    {
        $O_PARAM->Set('__wb_function','MultiBases');
        # Initialize $O_WB
        my $desc_file = '../xml/layout_multi.xml';
        $O_WB = New WebBuilder($O_PARAM,"",$desc_file);
        $O_WB->UnsetBehaviour('use_session');
        $O_WB->UnsetBehaviour('encoded_url')
    }
}

use strict;
use warnings;

use XML::Simple;
#use WebBuilder;
use Data::Dumper;
use Socket;
#use Logger;

use O2DBI::O2DBI_WebUtils;
use O2DBI::UpdateTable;
use O2DBI::UpdateTable::UpdateSouche;
use O2DBI::UpdateTable::UpdatePlasmide;
use O2DBI::UpdateTable::UpdateBiblio;
use O2DBI::UpdateTable::UpdateGene;
use O2DBI::UpdateTable::UpdateCaractere;
use O2DBI::UpdateTable::UpdateCaracterea;
use O2DBI::UpdateTable::UpdateCaracterep;
use O2DBI::UpdateTable::UpdateEspece;
use O2DBI::UpdateTable::UpdateRevue;
use O2DBI::UpdateTable::UpdateTransposon;
use O2DBI::UpdateTable::UpdateRestriction;
use clism::utilisateur;

use JSON;

$CGI::CLOSE_UPLOAD_FILES = 1;

MAIN:
{

    if ($O_PARAM->Get('__wb_function') eq 'Lang')
    {
        $O_WB->Realize(2);
    }
    else
    {
        $O_WB->Realize();
    }
}

# FONCTIONS APPELEES PAR Realize()
sub MainScreen
{
    my ($o_wb) = (@_);
    &PrintLoginInfo($o_wb);
    my $o_tmpl = &O2DBI::O2DBI_WebUtils::GetTemplate('welcome');
    my $doc_url= $O_CFG->Get('HTTP_ROOT').'/'.$O_CFG->Get('HTTP_DOC_DIR');
    $o_tmpl->AddVariable('DOC_URL',$doc_url);
    $o_tmpl->Realize();
}
sub Help
{
    my ($o_wb) = (@_);
    &PrintLoginInfo($o_wb);
    my $o_tmpl = &O2DBI::O2DBI_WebUtils::GetTemplate('help');
    $o_tmpl->Realize();
}
# Menu Login
sub Login
{
    my $o_tmpl = &O2DBI::O2DBI_WebUtils::GetTemplate('ecran_login');
    $o_tmpl->AddObject('PARAM',$O_PARAM);
    $o_tmpl->Realize();
}

# Menu Lang
sub Lang
{
    my ($o_wb) = (@_);
    
    my $o_param = $o_wb->GetObjectParam();
    $o_param->Set('__wb_lang',($o_param->Get('__wb_lang') eq 'fr') ? 'en' : 'fr');
    $o_param->Delete('__wb_function');
    $o_param->Delete('__wb_main_menu');
    my $url = $o_wb->GetUrl();
    print "Status: 302 Moved\n";
    print "Location: $url\n\n"; 
}
            
# Menu Update
sub Update
{
    my ($o_wb) = (@_);

    &PrintLoginInfo($o_wb);
    my $o_tmpl = &O2DBI::O2DBI_WebUtils::GetTemplate('ecran_update');
    my $doc_url= $O_CFG->Get('HTTP_ROOT').'/'.$O_CFG->Get('HTTP_DOC_DIR');
    
    $o_tmpl->AddVariable('DOC_URL',$doc_url);
    $o_tmpl->Realize();
}

# Menu Admin
sub Admin
{
    my ($o_wb) = (@_);

    &PrintLoginInfo($o_wb);
    my $o_tmpl = &O2DBI::O2DBI_WebUtils::GetTemplate('ecran_admin');
    my $doc_url= $O_CFG->Get('HTTP_ROOT').'/'.$O_CFG->Get('HTTP_DOC_DIR');
    
    $o_tmpl->AddVariable('DOC_URL',$doc_url);
    $o_tmpl->Realize();
}

# Menu Update
sub InsertStrain
{
    my ($o_wb) = (@_);
 
    &PrintLoginInfo($o_wb);
    my $o_insert_souche = New UpdateSouche($O_CFG,$O_LOGGER,$O_PARAM);
    $o_insert_souche->UpdateTable($o_wb);
}

sub ModifyStrain
{
    my ($o_wb) = (@_);

    &PrintLoginInfo($o_wb);
    my $o_modify_souche = New UpdateSouche($O_CFG,$O_LOGGER,$O_PARAM);
    $o_modify_souche->UpdateExistingRow($o_wb);
}

sub InsertPlasmid
{
    my ($o_wb) = (@_);
 
    &PrintLoginInfo($o_wb);
    my $o_insert_plasmide = New UpdatePlasmide($O_CFG,$O_LOGGER,$O_PARAM);
    $o_insert_plasmide->UpdateTable($o_wb);
}

sub ModifyPlasmid
{
    my ($o_wb) = (@_);
 
    &PrintLoginInfo($o_wb);
    my $o_insert_plasmide = New UpdatePlasmide($O_CFG,$O_LOGGER,$O_PARAM);
    $o_insert_plasmide->UpdateExistingRow($o_wb);
}

sub InsertBiblio
{
    my ($o_wb) = (@_);
 
    &PrintLoginInfo($o_wb);
    my $o_insert_biblio = New UpdateBiblio($O_CFG,$O_LOGGER,$O_PARAM);
    $o_insert_biblio->UpdateTable($o_wb);
}

sub ModifyBiblio
{
    my ($o_wb) = (@_);

    &PrintLoginInfo($o_wb);
    my $o_modify_biblio = New UpdateBiblio($O_CFG,$O_LOGGER,$O_PARAM);
    $o_modify_biblio->UpdateExistingRow($o_wb);
}

# Menus Souche, Plasmide, Biblio
sub Souche
{
    my ($o_wb) = (@_);
    
    &PrintLoginInfo($o_wb);
    my $o_display_souche = New DisplaySouche($O_CFG,$O_LOGGER,$O_PARAM);
    $o_display_souche->PrintFormTable($o_wb);
}

sub Plasmide
{
    my ($o_wb) = (@_);

    &PrintLoginInfo($o_wb);
    my $o_display_plasmide = New DisplayPlasmide($O_CFG,$O_LOGGER,$O_PARAM);
    $o_display_plasmide->PrintFormTable($o_wb);
}

sub Biblio
{
    my ($o_wb) = (@_);

    &PrintLoginInfo($o_wb);
    my $o_display_biblio = New DisplayBiblio($O_CFG,$O_LOGGER,$O_PARAM);
    $o_display_biblio->PrintFormTable($o_wb);
}

sub AdminGene
{
    my ($o_wb) = (@_);
 
    &PrintLoginInfo($o_wb);
    my $o_insert_gene = New UpdateGene($O_CFG,$O_LOGGER,$O_PARAM);
    $o_insert_gene->UpdateSecondaryTable($o_wb);
}    

sub AdminCaractere
{
    my ($o_wb) = (@_);
 
    &PrintLoginInfo($o_wb);
    my $o_insert_caractere = New UpdateCaractere($O_CFG,$O_LOGGER,$O_PARAM);
    $o_insert_caractere->UpdateSecondaryTable($o_wb);
}

sub AdminCaracterea
{
    my ($o_wb) = (@_);
 
    &PrintLoginInfo($o_wb);
    my $o_insert_caracterea = New UpdateCaracterea($O_CFG,$O_LOGGER,$O_PARAM);
    $o_insert_caracterea->UpdateSecondaryTable($o_wb);
}

sub AdminCaracterep
{
    my ($o_wb) = (@_);
 
    &PrintLoginInfo($o_wb);
    my $o_insert_caracterep = New UpdateCaracterep($O_CFG,$O_LOGGER,$O_PARAM);
    $o_insert_caracterep->UpdateSecondaryTable($o_wb);
}

sub AdminEspece
{
    my ($o_wb) = (@_);
 
    &PrintLoginInfo($o_wb);
    my $o_insert_espece = New UpdateEspece($O_CFG,$O_LOGGER,$O_PARAM);
    $o_insert_espece->UpdateSecondaryTable($o_wb);
}

sub AdminRevue
{
    my ($o_wb) = (@_);
 
    &PrintLoginInfo($o_wb);
    my $o_insert_revue = New UpdateRevue($O_CFG,$O_LOGGER,$O_PARAM);
    $o_insert_revue->UpdateSecondaryTable($o_wb);
}

sub AdminTransposon
{
    my ($o_wb) = (@_);
 
    &PrintLoginInfo($o_wb);
    my $o_insert_transposon = New UpdateTransposon($O_CFG,$O_LOGGER,$O_PARAM);
    $o_insert_transposon->UpdateSecondaryTable($o_wb);
}

sub AdminRestriction
{
    my ($o_wb) = (@_);
 
    &PrintLoginInfo($o_wb);
    my $o_insert_restriction = New UpdateRestriction($O_CFG,$O_LOGGER,$O_PARAM);
    $o_insert_restriction->UpdateSecondaryTable($o_wb);
}

sub PrintMaintenance
{
	#&Authentification();
	my $session_param = &O2DBI::O2DBI_WebUtils::UrlSessionParameters($O_PARAM);
	
    print STDOUT $O_CGI->h1("Curation");
    print STDOUT $O_CGI->p(
                   $O_CGI->i($O_PARAM->Get('schema_error') . ' ' . "Using schema: " . $O_PARAM->Get('schema_used')));
#	print STDOUT "<a href=\"index.cgi?__wb_schema=debug0\">Version non nettoyee</a><hr>";
#	print STDOUT "<a href=\"index.cgi?__wb_schema=public\">Version nettoyee</a><hr>";
	
    print STDOUT "<a href=\"caractere.cgi?$session_param\">table caractere</a><hr />";
    print STDOUT "<a href=\"caracterea.cgi?$session_param\">table caracterea</a><hr />";
    print STDOUT "<a href=\"caracterep.cgi?$session_param\">table caracterep</a><hr />";
    print STDOUT "<a href=\"transposon.cgi?$session_param\">table transposon</a><hr />";
    print STDOUT "<a href=\"gene.cgi?$session_param\">table gene</a><hr />";
    print STDOUT "<a href=\"nom_souche.cgi?$session_param\">table nom_souche</a><hr />";
    print STDOUT "<a href=\"nom_plasmide.cgi?$session_param\">table nom_plasmide</a><hr />";
    print STDOUT "<a href=\"siterestriction.cgi?$session_param\">table siterestriction</a><hr />";
    
}


sub AutocompleteFunction
{
    my ($o_wb)  = (@_);
    my $o_param = $o_wb->GetObjectParam();
    
    my $key      = $o_param->Get('__wb_autocomplete_field');
    $key         = 'field:'.$key if ($key !~ /field:/);
    my $o_dspl   = New DisplayInputField($main::O_CFG,$main::O_LOGGER,$o_param,undef);
    
    my $ra_values = $o_dspl->GetAutocompleteValues($key);
    
    # force a quote around the elements, even if we have only numbers
    ###my @a_qval = map(' '.$_,@$ra_values);

    my $json = new JSON;
    #my $js = $json->objToJson($ra_values);
    my $js = $json->encode($ra_values);
    print $js;
}

sub PrintLoginInfo
{
    my ($o_wb)  = (@_);
    
    my $o_param = $o_wb->GetObjectParam();
    my $login = ($O_PARAM->IsDefined('__wb_login')) ? $O_PARAM->Get('__wb_login') : $O_CFG->Get('login');
    
    if (!$o_param->IsDefined('__wba_ajax') or $o_param->Get('__wba_ajax') eq '0')
    {
        if ($login ne 'none')
        {
            $login =~ s/_guest$//;
            my $o_user = clism::utilisateur->init_login($login);
            my $o_tmpl = &O2DBI::O2DBI_WebUtils::GetTemplate('infos_login');
            $o_tmpl->AddObject('CFG',$O_CFG);
            $o_tmpl->AddObject('USER',$o_user) if ($o_user ne "");
            $o_tmpl->Realize();
        }
        else
        {
            print "<strong>" . $O_CFG->Get('CLISM_DESC') . "</strong>";
        }
    }
}

# Appele lorsque clism.cfg n'est pas trouve, on entre alors en mode MultiBases
sub MultiBases
{
    my ($o_wb) = (@_);
    
    my $url = $o_wb->GetCgi();
    
    #print "<pre>".join("\n",Dumper \%ENV)."</pre>";
    chdir '../cfg';
    print '<ul>';
    foreach my $f (glob('*.cfg'))
    {
        my ($nom) = ($f =~ /clism_(\w+)\./);
        if (defined $nom and $nom ne 'distrib')
        {
            my $o_cfg = New ParamParser($f);
            my $desc  = $o_cfg->Get('CLISM_DESC');
            my $mail  = $o_cfg->Get('CLISM_MAIL');
            
            print "<li><strong><a href=\"$url?__wb_nom=$nom\" >$nom</a> </strong>$desc ($mail)</li>";
        }
    }
    print "</ul>";
}
