# 
#       Sebastien.Letort@toulouse.inra.fr
#       Created: August 30, 2007
#       Last Updated: August 30, 2007
#       $Id: Enum.pm 348 2008-04-21 15:38:46Z sletort $
#

package Enum;

=pod

=head1 NAME

	Enum

=head1 SYNOPSIS

	# comme constante !
	my $_o_ALLOW_FIELDS = New Enum( 'un', 'deux', 'trois');
	my $val = 'deux';
	print 'correct'	if( $_o_ALLOW_FIELDS->IsOk( $val ) );
	$val = 'quatre';
	print 'inconnu'	if( $_o_ALLOW_FIELDS->IsOk( $val ) );

=head1 DESCRIPTION

	This is an object way of putting enum data.
	The other way is to use $hash{$val}

=cut


use strict;
use warnings;

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 348 $ =~ /\d+/g); $r[0]};
}

=head2 Function New

 Title        : New
 Usage        : my $o_enum = New Enum( 'un', 'dexu', 'ttt' );
 Prerequisite : none
 Function     : constructor
 Args         : an array of vals, anything
 Globals      : none

=cut

sub New
{
	my $class = shift;
	my $self = {};
	bless( $self, $class );

	foreach my $val (@_)
	{	$self->{ $val } = 1;	}

	return $self;
}

=head2 Function IsOk

 Title        : IsOk
 Usage        : Trust()	if $o_enum->IsOk( $value );
 Prerequisite : none
 Function     : return true if $value is part of the Enum allowed value
 Return       : 1 if $value is part of the Enum values, 0 otherwise
 Args         : $value, anything
 Globals      : none

 Note         : DEPRECATED use Includes

=cut

sub IsOk
{
	my $self = shift;
	my ( $val ) = @_;

	warn( "'IsOk' is deprecated, please change it to 'Includes'" );

	return	defined $self->{ $val } ? 1 : 0;
}

=head2 Function Includes

 Title        : Includes
 Usage        : Trust()	if $o_enum->Includes( $value );
 Prerequisite : none
 Function     : return true if $value is part of the Enum allowed value
 Return       : 1 if $value is part of the Enum values, 0 otherwise
 Args         : $value, anything
 Globals      : none

=cut

sub Includes
{
	my $self = shift;
	my ( $val ) = @_;

	return	defined $self->{ $val } ? 1 : 0;
}

=head2 Function ListItems

 Title        : ListItems
 Usage        : my @a_allowed_values = $o_enum->ListItems();
 Prerequisite : none
 Function     : return the values passed to the constructor
 Args         : none
 Return       :	an array of anything
 Globals      : none

=cut

sub ListItems
{
	my $self = shift;

	return	keys %$self;
}


=head2 function AddValues

 Title        : AddValues
 Usage        : $o_enum->AddValues( @a_values );
 Prerequisite : none
 Function     : add new allowed values
 Return       : $self
 Args         : @a_values, a list of values

=cut

sub AddValues
{
	my $self = shift;
	my ( @a_values ) = @_;

	foreach my $value ( @a_values )
	{
		$self->{ $value } = 1;
	}

	return $self;	# devrait être une procedure, chercher si utilise
}


=head2 Procedure RemoveValues

 Title        : RemoveValues
 Usage        : $o_enum->RemoveValues( @a_values );
 Prerequisite : none
 Procedure     : removed allowed values
 Args         : @a_values, a list of values

=cut

sub RemoveValues
{
	my $self = shift;
	my ( @a_values ) = @_;

	foreach my $value ( @a_values )
	{
		delete $self->{ $value };
	}

	return;
}



1;
