# Copyright INRA/CNRS
# 
# Emmanuel.Courcelle@toulouse.inra.fr
# Sebastien.Carrere@toulouse.inra.fr
# 
# Ce logiciel est un programme informatique servant à [rappeler les
# caractéristiques techniques de votre logiciel]. 
# 
# Ce logiciel est régi par la licence [CeCILL|CeCILL-B|CeCILL-C] soumise au droit français et
# respectant les principes de diffusion des logiciels libres. Vous pouvez
# utiliser, modifier et/ou redistribuer ce programme sous les conditions
# de la licence [CeCILL|CeCILL-B|CeCILL-C] telle que diffusée par le CEA, le CNRS et l'INRIA 
# sur le site "http://www.cecill.info".
# 
# En contrepartie de l'accessibilité au code source et des droits de copie,
# de modification et de redistribution accordés par cette licence, il n'est
# offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
# seule une responsabilité restreinte pèse sur l'auteur du programme,  le
# titulaire des droits patrimoniaux et les concédants successifs.
# 
# A cet égard  l'attention de l'utilisateur est attirée sur les risques
# associés au chargement,  à l'utilisation,  à la modification et/ou au
# développement et à la reproduction du logiciel par l'utilisateur étant 
# donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
# manipuler et qui le réserve donc à des développeurs et des professionnels
# avertis possédant  des  connaissances  informatiques approfondies.  Les
# utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
# logiciel à leurs besoins dans des conditions permettant d'assurer la
# sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
# à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
# 
# Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
# pris connaissance de la licence [CeCILL|CeCILL-B|CeCILL-C], et que vous en avez accepté les
# termes.

package DisplayField;

use LipmObject;
use base qw(LipmObject);

=head2 Function _Init (the constructor)
 Title            : _Init
 Usage            : my $o_obj = New DisplayTable($o_cfg, $o_logger)
 Function         : threw the New function of LipmObject, builds the object
 Returns          : The object
 Args             : $o_cfg   = A ParamParser containing the configuration parameters
                    $o_logger= A Logger object (optional)
 Globals          : none

=cut
sub _Init
{
    my $self = shift;
    my ($o_cfg,$o_logger,$o_param,$rh_schema) = (@_);
    
    $self -> {_o_cfg}   = $o_cfg;
    $self -> SetLogger($o_logger) if (defined $o_logger);
    $self -> {_o_param}   = $o_param;
    $self -> {_rh_schema} = $rh_schema;
    return $self;
}

=head2 Procedure SetRow
 Title            : SetRow
 Usage            : $o_obj->SetRow($rh_row}
 Procedure        : Set the current row, ie the data structure ($rh_) from which most data will be searched
 Args             : $rh_row = A "row" (an sql request is an array of row, this one of those rows)
 Globals          : none

=cut
sub SetRow
{
    my $self = shift;
    
    $self->{__rh_row} = @_[0];
}

=head2 Fonction GetHtml
 Title    : Gethtml
 Usage    : Pour utiliser a partir de AnotherTemplate
 Function : Renvoie en html la valeur correspondant a la cle passee. Cette cle doit figurer dans __rh_row
            Si la valeur est un tableau, on renvoie les resultats separes par des paragraphes
 Args     : La cle a afficher
 Globals  : none

=cut
sub GetHtml
{
    my $self = shift;

    my ($key) = (@_);
    my $value = $self->{__rh_row}{$key};
    return '' if (!defined $value);
    if (ref $value eq 'ARRAY')
    {
        my $rvl = (@$value == 0) ? '' : '<p>'.join('</p><p>',@$value).'</p>';
        $rvl =~ s|<p></p>|<p>&nbsp;</p>|g;  # avoiding <p></p>
        return $rvl;
    }
    else
    {
        return $value;
    }
}

=head2 Fonction GetHtmlDelete
 Title    : GetHtmlDelete
 Usage    : Pour utiliser a partir de AnotherTemplate
 Function : Pareil que GetHtml, mais renvoie en plus une checkbox pour permettre de cocher les valeurs a supprimer
 Args     : La cle a afficher
 Globals  : none

=cut
sub GetHtmlDelete
{
    my $self = shift;
    
    my ($key,$label_keys) = (@_);
    my $value = $self->{__rh_row}{$key};
    
    my @a_labels = ();
    foreach my $label_key (split (',',$label_keys))
    {
	    push (@a_labels, $self->{__rh_row}{$label_key});
    }
    my $html = '';
    #warn "DEL $key $value";
    #use Data::Dumper;
    #warn "DEL ".Dumper ($self->{__rh_row});
    #warn "DEL $key $value ".Dumper $value;
    if (ref $value eq 'ARRAY')
    {
        my $f=1;
        my $count = 0;
        foreach my $val (@{$value})
        {
            # $val may be an html string, we extract the only useful information (HOULALA quel bordel)
            if ($val =~ /input_code=(\d+)/)
            {
                $val = $1;
            }
            my $label_string = '';
            foreach my $label (@a_labels)
            {
                $label_string .= $label->[$count] . ' ';
            }
            $html .= "<p><input type=\"checkbox\" name=\"fieldD${f}:${key}\" value=\"${val}\" />$label_string </p>";
            
            $count++;
            $f++;
        }
        return $html;
    }
    else
    {
        
        return $value;
    }
}


1;

