# Copyright INRA/CNRS
# 
# Emmanuel.Courcelle@toulouse.inra.fr
# Sebastien.Carrere@toulouse.inra.fr
# 
# Ce logiciel est un programme informatique servant à [rappeler les
# caractéristiques techniques de votre logiciel]. 
# 
# Ce logiciel est régi par la licence [CeCILL|CeCILL-B|CeCILL-C] soumise au droit français et
# respectant les principes de diffusion des logiciels libres. Vous pouvez
# utiliser, modifier et/ou redistribuer ce programme sous les conditions
# de la licence [CeCILL|CeCILL-B|CeCILL-C] telle que diffusée par le CEA, le CNRS et l'INRIA 
# sur le site "http://www.cecill.info".
# 
# En contrepartie de l'accessibilité au code source et des droits de copie,
# de modification et de redistribution accordés par cette licence, il n'est
# offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
# seule une responsabilité restreinte pèse sur l'auteur du programme,  le
# titulaire des droits patrimoniaux et les concédants successifs.
# 
# A cet égard  l'attention de l'utilisateur est attirée sur les risques
# associés au chargement,  à l'utilisation,  à la modification et/ou au
# développement et à la reproduction du logiciel par l'utilisateur étant 
# donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
# manipuler et qui le réserve donc à des développeurs et des professionnels
# avertis possédant  des  connaissances  informatiques approfondies.  Les
# utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
# logiciel à leurs besoins dans des conditions permettant d'assurer la
# sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
# à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
# 
# Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
# pris connaissance de la licence [CeCILL|CeCILL-B|CeCILL-C], et que vous en avez accepté les
# termes.

package DisplayField::BiblioField;

use strict;
use WebBuilder;

use O2DBI::DisplayField;
use O2DBI::DisplayField::PlasmideField;
use O2DBI::DisplayField::SoucheField;

use base qw(DisplayField);
use clism::DBMS;

=head2 Fonction GetHtmlTitle
 Title    : Gethtmltitle
 Usage    : Pour utiliser a partir de AnotherTemplate
 Function : Renvoie en html la valeur correspondant a la cle passee (un titre d'article) mais tronque le titre a 60 caracteres
 Args     : La cle a afficher
 Globals  : none

=cut
sub GetHtmlTitle
{
    my $self = shift;

    my ($key) = (@_);
    my $value = $self->{__rh_row}{$key};
    if (ref $value eq 'ARRAY')
    {
        $value = $value->[0];
    }
    $value = substr($value, 0, 56) . '...' if (length ($value) >= 60);
    return $value;
}

=head2 Function GetCodeNomSouche, GetCodeNomPlasmide
 Title        : GetCodeNomSouche, GetCodeNomPlasmide
 Usage        : A partir de anotherTemplate
 Function     : Lance une requete sql pour recuperer les infos sur les plasmides lies a cette reference biblio
 Returns      : $html_text : HTML to be displayed (quelques lignes de tableau)
 Args         : $key 
=cut
sub GetCodeNomSouche
{
    my $self = shift;
    
    my ($key)       = (@_);
    my $id_biblio = $self->{__rh_row}{$key};
    my $o_param     = $self->{_o_param};
    
	my $sql_query = "select souche.code,nom_souche.nom from biblio,souche,biblio_souche,nom_souche "
			. "where biblio.id=$id_biblio and biblio.id = biblio_souche.biblio_id and souche.id = biblio_souche.souche_id "
			. "and biblio_souche.souche_id = nom_souche.souche_id ";

    my $sth = $clism_DBH->prepare(qq {
					$sql_query	
						});
						
	$sth->execute;
	my $rh_codes =  {};
	while ((my $code,my $nom) = $sth->fetchrow_array)
	{
		$rh_codes->{$code} = [] if (! defined $rh_codes->{$code});
		push (@{$rh_codes->{$code}},$nom);
	}
	
	my $html_code = '';
	my $cgi_target_script ='?__wb_login='.$o_param->Get('__wb_login');
    $cgi_target_script .= '&amp;__wb_session='.$o_param->Get('__wb_session');
    $cgi_target_script .= '&amp;__wb_function=Souche&amp;__wb_main_menu=Souche';
    $cgi_target_script .= '&amp;__wb_nom='.$o_param->Get('__wb_nom').'&amp;field:souche.code=';
	
	foreach my $code (keys %{$rh_codes})
	{
		$html_code .= "<tr><td><a href=\"$cgi_target_script$code\" >$code</a></td><td><i>" . join (',', @{$rh_codes->{$code}}) . "</i></td></tr>";
	
	}
	return $html_code;
}
sub GetCodeNomPlasmide
{
    my $self = shift;
    
    my ($key)       = (@_);
    my $id_biblio = $self->{__rh_row}{$key};
    my $o_param     = $self->{_o_param};
    
	my $sql_query = "select plasmide.code,nom_plasmide.nom from biblio,plasmide,biblio_plasmide,nom_plasmide "
			. "where biblio.id=$id_biblio and biblio.id = biblio_plasmide.biblio_id and plasmide.id = biblio_plasmide.plasmide_id "
			. "and biblio_plasmide.plasmide_id = nom_plasmide.plasmide_id ";

    my $sth = $clism_DBH->prepare(qq {
					$sql_query	
						});
						
	$sth->execute;
	my $rh_codes =  {};
	while (my ($code,$nom) = $sth->fetchrow_array)
	{
		$rh_codes->{$code} = [] if (! exists $rh_codes->{$code});
		push (@{$rh_codes->{$code}},$nom);  
	}
	
	my $html_code = '';
	my $cgi_target_script ='?__wb_login='.$o_param->Get('__wb_login');
    $cgi_target_script .= '&amp;__wb_session='.$o_param->Get('__wb_session');
    $cgi_target_script .= '&amp;__wb_function=Plasmide&amp;__wb_main_menu=Plasmide';
    $cgi_target_script .= '&amp;__wb_nom='.$o_param->Get('__wb_nom').'&amp;field:plasmide.code=';
	
	foreach my $code (keys %{$rh_codes})
	{
		$html_code .= "<tr><td><a href=\"$cgi_target_script$code\" >$code</a></td><td><i>" . join (',', @{$rh_codes->{$code}}) . "</i></td></tr>";
	
	}
	return $html_code;
}
1;

