# Copyright INRA/CNRS
# 
# Emmanuel.Courcelle@toulouse.inra.fr
# Sebastien.Carrere@toulouse.inra.fr
# 
# Ce logiciel est un programme informatique servant à [rappeler les
# caractéristiques techniques de votre logiciel]. 
# 
# Ce logiciel est régi par la licence [CeCILL|CeCILL-B|CeCILL-C] soumise au droit français et
# respectant les principes de diffusion des logiciels libres. Vous pouvez
# utiliser, modifier et/ou redistribuer ce programme sous les conditions
# de la licence [CeCILL|CeCILL-B|CeCILL-C] telle que diffusée par le CEA, le CNRS et l'INRIA 
# sur le site "http://www.cecill.info".
# 
# En contrepartie de l'accessibilité au code source et des droits de copie,
# de modification et de redistribution accordés par cette licence, il n'est
# offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
# seule une responsabilité restreinte pèse sur l'auteur du programme,  le
# titulaire des droits patrimoniaux et les concédants successifs.
# 
# A cet égard  l'attention de l'utilisateur est attirée sur les risques
# associés au chargement,  à l'utilisation,  à la modification et/ou au
# développement et à la reproduction du logiciel par l'utilisateur étant 
# donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
# manipuler et qui le réserve donc à des développeurs et des professionnels
# avertis possédant  des  connaissances  informatiques approfondies.  Les
# utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
# logiciel à leurs besoins dans des conditions permettant d'assurer la
# sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
# à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
# 
# Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
# pris connaissance de la licence [CeCILL|CeCILL-B|CeCILL-C], et que vous en avez accepté les
# termes.

package DisplayField::PlasmideField;

use strict;
use WebBuilder;

use O2DBI::DisplayField;
use base qw(DisplayField);
use clism::DBMS;
use MIME::Base64;

=head2 Function GetAntibio
 Title        : Getantibio
 Usage        : A partir de anotherTemplate
 Function     : Lance une requete sql pour recuperer les infos sur les antibios de ce plasmide
 Returns      : $html_text : HTML to be displayed (quelques lignes de tableau)
 Args         : $key 
=cut

sub GetAntibio
{
    my $self = shift;
    
    my ($key)       = (@_);
    my $id_plasmide = $self->{__rh_row}{$key};
    
	my $sql_query = "select caracterep.nom,phenotypeconfere.concentration,phenotypeconfere.commentaire "
    . "from phenotypeconfere, caracterep "
    . "where phenotypeconfere.plasmide_id = $id_plasmide "
    . "and caracterep.antibio = 't' and phenotypeconfere.caracterep_id = caracterep.id ";
    
	my $sth = $clism_DBH->prepare(qq { $sql_query });
    $sth->execute;
	
	my $html_code = "";
	while (my ($antibio,$concentration,$commentaire) = $sth->fetchrow_array)
	{
        $html_code .= "<tr><td>$antibio</td><td>$concentration</td><td>$commentaire</td></tr>\n";
	}
	return $html_code;

}

=head2 Function GetPlasmidMap
 Title        : GetPlasmideMap
 Usage        : A partir de anotherTemplate
 Function     : recupere les fichiers jpg et gb de la carte de plasmide
                cree les fichiers dans le repertoire de session
 Returns      : $html_text : HTML to be displayed (une ligne de tableau)
 Args         : $key 
=cut
sub GetPlasmidMap
{
    my $self = shift;
    
    my ($key)       = (@_);
    my $rh_row      = $self->{__rh_row};
    my $id_plasmide = $rh_row->{$key};
    
    my $o_cfg   = $self->{_o_cfg};
    my $o_param = $self->{_o_param};
    
    my $http_root   = $o_cfg->Get('SERVERNAME').'/'.$o_cfg->Get('HTTP_ROOT');
    my $tmp_url     = $http_root.$o_cfg->Get('HTTP_TMP_DIR').'/'.$o_param->Get('__wb_session');

    my $tmp_file_jpg= 'vnti_jpg.'.$id_plasmide.'.jpg';
    my $tmp_file_gb = 'vnti_gb.'.$id_plasmide.'.gb';
    my $jpg_content = MIME::Base64::decode($rh_row->{vnti_jpg});
    my $gb_content  = MIME::Base64::decode($rh_row->{vnti_gb});

    my $html_code   = "";
    if ($jpg_content ne '')
    {
        #warn "koukou pas vide ".$self->{__rh_row}{code};
        my $fh_tmp_file_jpg = new IO::File(">$tmp_file_jpg") or croak ("$!:$tmp_file_jpg");
        print $fh_tmp_file_jpg $jpg_content;
        $fh_tmp_file_jpg = "";

        my $fh_tmp_file_gb = new IO::File(">$tmp_file_gb") or croak ("$!:$tmp_file_gb");
        print $fh_tmp_file_gb $gb_content;
        $fh_tmp_file_jpg = "";

        $html_code  = "<tr><td><a class=\"MapPopup\" href=\"$tmp_url/$tmp_file_jpg\"><img src=\"$tmp_url/$tmp_file_jpg\" width=\"80\" alt=\"$id_plasmide plasmid map\" /></a></td>";
        $html_code .= "<td><a href=\"$tmp_url/$tmp_file_gb\">data file</a></td></tr>";
    }

    return $html_code;
}

=head2 Function GetCodeNomSouche,GetTitrebiblio
 Title        : GetCodeNomSouche,GetTitrebiblio
 Usage        : A partir de anotherTemplate
 Function     : Lance une requete sql pour recuperer les infos sur les transposons issus des plasmides
 Returns      : $html_text : HTML to be displayed (quelques lignes de tableau)
 Args         : $key 
=cut
sub GetCodeNomSouche
{
    my $self = shift;
    
    my ($key)       = (@_);
    my $id_plasmide = $self->{__rh_row}{$key};
    my $o_param     = $self->{_o_param};
    
	my $sql_query = "select souche.code,nom_souche.nom from plasmide,souche,plasmide_souche,nom_souche "
			. "where plasmide.id=$id_plasmide and plasmide.id = plasmide_souche.plasmide_id "
			. "and plasmide_souche.souche_id = nom_souche.souche_id and souche.id = plasmide_souche.souche_id ";

    my $sth = $clism_DBH->prepare(qq {
					$sql_query	
						});
						
	$sth->execute;
	my $rh_codes =  {};
	while ((my $code,my $nom) = $sth->fetchrow_array)
	{
		$rh_codes->{$code} = [] if (! defined $rh_codes->{$code});
		push (@{$rh_codes->{$code}},$nom);
	}
	
	my $html_code = '';
	my $cgi_target_script ='?__wb_login='.$o_param->Get('__wb_login');
    $cgi_target_script .= '&amp;__wb_session='.$o_param->Get('__wb_session');
    $cgi_target_script .= '&amp;__wb_function=Souche&amp;__wb_main_menu=Souche';
    $cgi_target_script .= '&amp;__wb_nom='.$o_param->Get('__wb_nom').'&amp;field:souche.code=';
	
	foreach my $code (keys %{$rh_codes})
	{
		$html_code .= "<a href=\"$cgi_target_script$code\">$code</a> <i>" . join (',', @{$rh_codes->{$code}}) . "</i><br />";
	
	}
	return $html_code;
}
sub GetTitreBiblio
{
    my $self = shift;
    
    my ($key)       = (@_);
    my $id_plasmide = $self->{__rh_row}{$key};
    my $o_param     = $self->{_o_param};
    #    return $id_plasmide;
    
	my $sql_query = "select biblio.titre,biblio.id from biblio,plasmide,biblio_plasmide "
			. "where plasmide.id=$id_plasmide and biblio.id = biblio_plasmide.biblio_id and plasmide.id = biblio_plasmide.plasmide_id ";

    my $sth = $clism_DBH->prepare(qq {
					$sql_query	
						});
						
	$sth->execute;
	
	my $html_code = '';
	my $cgi_target_script ='?__wb_login='.$o_param->Get('__wb_login');
    $cgi_target_script .= '&amp;__wb_session='.$o_param->Get('__wb_session');
    $cgi_target_script .= '&amp;__wb_function=Biblio&amp;__wb_main_menu=Biblio';
    $cgi_target_script .= '&amp;__wb_nom='.$o_param->Get('__wb_nom').'&amp;field:biblio.id=';
	
	while ((my $titre,my $id) = $sth->fetchrow_array)
	{
        $html_code .= "<a href=\"$cgi_target_script$id\">$titre</a><br />";
	}
	return $html_code;
}
1;

