# Copyright INRA/CNRS
# 
# Emmanuel.Courcelle@toulouse.inra.fr
# Sebastien.Carrere@toulouse.inra.fr
# 
# Ce logiciel est un programme informatique servant à [rappeler les
# caractéristiques techniques de votre logiciel]. 
# 
# Ce logiciel est régi par la licence [CeCILL|CeCILL-B|CeCILL-C] soumise au droit français et
# respectant les principes de diffusion des logiciels libres. Vous pouvez
# utiliser, modifier et/ou redistribuer ce programme sous les conditions
# de la licence [CeCILL|CeCILL-B|CeCILL-C] telle que diffusée par le CEA, le CNRS et l'INRIA 
# sur le site "http://www.cecill.info".
# 
# En contrepartie de l'accessibilité au code source et des droits de copie,
# de modification et de redistribution accordés par cette licence, il n'est
# offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
# seule une responsabilité restreinte pèse sur l'auteur du programme,  le
# titulaire des droits patrimoniaux et les concédants successifs.
# 
# A cet égard  l'attention de l'utilisateur est attirée sur les risques
# associés au chargement,  à l'utilisation,  à la modification et/ou au
# développement et à la reproduction du logiciel par l'utilisateur étant 
# donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
# manipuler et qui le réserve donc à des développeurs et des professionnels
# avertis possédant  des  connaissances  informatiques approfondies.  Les
# utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
# logiciel à leurs besoins dans des conditions permettant d'assurer la
# sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
# à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
# 
# Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
# pris connaissance de la licence [CeCILL|CeCILL-B|CeCILL-C], et que vous en avez accepté les
# termes.

package DisplayField::SoucheField;

use strict;
use WebBuilder;

use O2DBI::DisplayField;
use base qw(DisplayField);
use clism::DBMS;

=head2 Function GetAntibio
 Title        : Getantibio
 Usage        : A partir de anotherTemplate
 Function     : Lance une requete sql pour recuperer les infos sur les antibios de ce plasmide
 Returns      : $html_text : HTML to be displayed (quelques lignes de tableau)
 Args         : $key 
=cut

sub GetAntibio
{
    my $self = shift;
    
    my ($key)     = (@_);
    my $id_souche = $self->{__rh_row}{$key};
    
	my $sql_query = "select caracterep.nom,antibios.concentration,antibios.commentaire "
    . "from antibios, caracterep "
    . "where antibios.souche_id = $id_souche "
    . "and caracterep.antibio = 't' and antibios.caracterep_id = caracterep.id ";
    
	my $sth = $clism_DBH->prepare(qq { $sql_query });
    $sth->execute;
	
	my $html_code = "";
	while (my ($antibio,$concentration,$commentaire) = $sth->fetchrow_array)
	{
        $html_code .= "<tr><td>$antibio</td><td>$concentration</td><td>$commentaire</td></tr>\n";
	}
    
	$sql_query = "select plasmide.code,caracterep.nom,phenotypeconfere.concentration,phenotypeconfere.commentaire "
			. "from phenotypeconfere, caracterep,plasmide_souche,plasmide "
			. "where plasmide_souche.souche_id= $id_souche and plasmide_souche.plasmide_id = plasmide.id and phenotypeconfere.plasmide_id = plasmide_souche.plasmide_id "
			. "and caracterep.antibio = 't' and phenotypeconfere.caracterep_id = caracterep.id ";

    $sth = $clism_DBH->prepare(qq {
            $sql_query	
    });
    
    $sth->execute;	
	
	while ((my $plasmide,my $antibio,my $concentration,my $commentaire) = $sth->fetchrow_array)
	{
        $concentration = "&nbsp;" if ($concentration eq "");
        $commentaire = "&nbsp;" if ($commentaire eq "");
		$html_code .= "<tr><td><i>$antibio ($plasmide)</i></td><td><i>$concentration</i></td><td><i>$commentaire</i></td></tr>\n";
	}
	return $html_code;
}
sub GetAntibiosDeleteCheckbox
{
    my $self = shift;
    
    my ($key)     = (@_);
    my $o_param = $self->{_o_param};
    my $id_souche =  $o_param->Get('field:souche.id');
    
    my $sql_query = "select caracterep.nom,antibios.concentration,antibios.commentaire,antibios.id "
    . "from antibios, caracterep "
    . "where antibios.souche_id = $id_souche "
    . "and antibios.caracterep_id = caracterep.id ";
    
    my $sth = $clism_DBH->prepare(qq { $sql_query });
    $sth->execute;
	
    my $html_code = "";
    my $count = 1;
    while (my ($antibio,$concentration,$commentaire,$antibio_id) = $sth->fetchrow_array)
    {
	#<p><input type=\"checkbox\" name=\"fieldD${f}:${key}\" value=\"${val}\" />$label_string </p>";
        $html_code .= "<tr><td>$antibio</td><td>$concentration</td><td>$commentaire</td><td><input type=\"checkbox\" name=\"fieldD$count:antibios.id\" value=\"$antibio_id\" /></td></tr>\n";
	$count ++;
    }
    return $html_code;
}


=head2 Function GetPhenotype
 Title        : GetPhenotype
 Usage        : A partir de anotherTemplate
 Function     : Lance une requete sql pour recuperer les infos sur les phenotypes de cette souche
 Returns      : $html_text : HTML to be displayed (quelques lignes de tableau)
 Args         : $key 
=cut

sub GetPhenotype
{
    my $self = shift;
    
    my ($key)     = (@_);
    my $id_souche = $self->{__rh_row}{$key};
    
	my $sql_query = "select distinct caracterep.nom "
    . "from antibios, caracterep "
    . "where antibios.souche_id = $id_souche "
    . "and antibios.caracterep_id = caracterep.id ";
    
	my $sth = $clism_DBH->prepare(qq { $sql_query });
    $sth->execute;
	
	my $html_code = "<ul>";
	while (my ($phenotype) = $sth->fetchrow_array)
	{
        $html_code .= "<li>$phenotype</li>\n";
	}
	$html_code .= "</ul>";
	
	return $html_code;
}

=head2 Function GetTransposonP
 Title        : GetTransposonP
 Usage        : A partir de anotherTemplate
 Function     : Lance une requete sql pour recuperer les infos sur les transposons issus des plasmides
 Returns      : $html_text : HTML to be displayed (quelques lignes de tableau)
 Args         : $key 
=cut

sub GetTransposonP
{
    my $self = shift;
    
    my ($key)     = (@_);
    my $id_souche = $self->{__rh_row}{$key};

    my $result = '';
    my $ra_o_plasmide_souche = &O2DBI::O2DBI_WebUtils::ExecuteFetchbySQL("souche_id = $id_souche", "plasmide_souche");

    foreach my $o_plasmide_souche (@{$ra_o_plasmide_souche})
    {
        my $ra_o_transposon_plasmide =
          &O2DBI::O2DBI_WebUtils::ExecuteFetchbySQL("plasmide_id = " . $o_plasmide_souche->{plasmide_id},
                                                    "transposon_plasmide");

        foreach my $o_transposon_plasmide (@{$ra_o_transposon_plasmide})
        {
            my $ra_o_transposon =
              &O2DBI::O2DBI_WebUtils::ExecuteFetchbySQL("id = " . $o_transposon_plasmide->{transposon_id},
                                                        "transposon");
            $result .= $ra_o_transposon->[0]->{nom} . ',';
        }

    }
    chop $result;

    return "$result";
}

=head2 Function GetCodeNomPlasmide,GetTitreBiblio
 Title        : GetCodeNomPlasmide,GetTitreBiblio
 Usage        : A partir de anotherTemplate
 Function     : Lance une requete sql pour recuperer les infos sur les transposons issus des plasmides
 Returns      : $html_text : HTML to be displayed (quelques lignes de tableau)
 Args         : $key 
=cut
sub GetCodeNomPlasmide
{
    my $self = shift;
    
    my ($key)     = (@_);
    my $id_souche = $self->{__rh_row}{$key};
    my $o_param   = $self->{_o_param};
    
	my $sql_query = "select plasmide.code,nom_plasmide.nom from plasmide,souche,plasmide_souche,nom_plasmide "
			. "where souche.id=$id_souche and souche.id = plasmide_souche.souche_id "
			. "and plasmide_souche.plasmide_id = nom_plasmide.plasmide_id and plasmide.id = plasmide_souche.plasmide_id ";
	use clism::DBMS;
	my $sth = $clism_DBH->prepare(qq {
					$sql_query	
						});
						
	$sth->execute;
	my $rh_codes =  {};
	while ((my $code,my $nom) = $sth->fetchrow_array)
	{
		$rh_codes->{$code} = [] if (! defined $rh_codes->{$code});
		push (@{$rh_codes->{$code}},$nom);
	}
	
	
	my $cgi_target_script ='?__wb_login='.$o_param->Get('__wb_login');
    $cgi_target_script .= '&amp;__wb_session='.$o_param->Get('__wb_session');
    $cgi_target_script .= '&amp;__wb_function=Plasmide&amp;__wb_main_menu=Plasmide';
    $cgi_target_script .= '&amp;__wb_nom='.$o_param->Get('__wb_nom').'&amp;field:plasmide.code=';
	my $html_code = '';
	foreach my $code (keys %{$rh_codes})
	{
		$html_code .= "<a class=\"BlankWin\" href=\"$cgi_target_script$code\"><i>" . join (',', @{$rh_codes->{$code}}) . "</i></a><br />";
	
	}
	return $html_code;
	

}
sub GetTitreBiblio
{
    my $self = shift;
    
    my ($key)     = (@_);
    my $id_souche = $self->{__rh_row}{$key};
    my $o_param   = $self->{_o_param};
    
	my $sql_query = "select biblio.titre,biblio.id from biblio,souche,biblio_souche "
			. "where souche.id=$id_souche and biblio.id = biblio_souche.biblio_id and souche.id = biblio_souche.souche_id ";

    my $sth = $clism_DBH->prepare(qq {
					$sql_query	
						});
						
	$sth->execute;
	
	my $html_code = '';
	my $cgi_target_script ='?__wb_login='.$o_param->Get('__wb_login');
    $cgi_target_script .= '&amp;__wb_session='.$o_param->Get('__wb_session');
    $cgi_target_script .= '&amp;__wb_function=Biblio&amp;__wb_main_menu=Biblio';
    $cgi_target_script .= '&amp;__wb_nom='.$o_param->Get('__wb_nom').'&amp;field:biblio.id=';
	
	while ((my $titre,my $id) = $sth->fetchrow_array)
	{
        $html_code .= "<a class=\"BlankWin\" href=\"$cgi_target_script$id\" >$titre</a><br />";
	}
	return $html_code;
}
1;

