#!/usr/bin/perl -I../lib

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# sebastien.carrere@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for managing strains and plasmids in the LIPM

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

package DisplayInputField;

=pod
=head1 NAME

	DisplayInputField

=head1 SYNOPSIS
    Derive de ManipInputField, utilisation avec AnotherTemplate
    
=head1 DESCRIPTION

    Cet objet derive de ManipInputField, il est utilise avec AnotherTemplate
    Il permet d'afficher les champs input en fonction du type de champ de la base de donnees,
    celui-ci est donne par le schema

=cut

use strict;
use warnings;

use O2DBI::O2DBI_WebUtils;
use CGI;
use base qw(LipmObject);
use Data::Dumper;

#
# Le constructeur
#
# Args = Le hash decrivant le schema de la base de donnees
#

sub _Init
{
    my $self = shift;
 
    my ($o_cfg,$o_logger,$o_param,$rh_schema) = (@_);
    
    $self -> {_o_cfg}   = $o_cfg;
    $self -> SetLogger($o_logger) if (defined $o_logger);
    
    $self->{__rh_schema} = $rh_schema;
    $self->{__o_cgi}     = new CGI;
    $self->{__o_param}   = $o_param;
    $self->{__edit_mode} = 0;
}

sub SetEditMode
{
    my $self = shift;
    
    $self->{__edit_mode} = 1;
}

=head2 Fonction GetHtml

 Title    : GetHtml
 Usage    : my $html = GetHtml('field:table.champ','field_type')
 Fonction : Doit etre utilisee via AnotherTemplate
            Ecrit un element de formulaire html, l'element (type et taille) depend du field_type
            Le field_type peut etre passe par parametre, sinon le type de la base de donnees est utilise
 Args     : $key   La cle, exemple field:souche.nom (field: est obligatoire !)
            $field_type (facultatif) le type du champ (ex. VARCHAR(10))
 Retourne : le code html
 Globals  : Aucun

=cut
sub GetHtml
{
    my $self = shift;
    my ($key, $field_type) = (@_);
    
    my $o_param = $self->{__o_param};
    
    my ($table,$field) = $self->__ParseKey($key);
    $field_type = O2DBI::O2DBI_WebUtils::GetTableFieldType($self->{__rh_schema},$table,$field) unless (defined $field_type);
    $self->_Trace("table: $table field: $field field_type: $field_type\n",'*','3');
    
    if (($field_type eq 'INTEGER') || ($field_type eq 'REAL'))
    {
        return $self->__GetHtmlNumber($key);
    }
    elsif ($field_type eq 'DATE')
    {
        return $self->__GetHtmlDate($key);
    }
    elsif ($field_type eq 'TEXT')
    {
        return $self->__GetHtmlText($key);
    }
    elsif ($field_type =~ /CHAR\((\d+)\)/)
    {
        return $self->__GetHtmlVarchar($key,$1);
    }
    elsif ($field_type eq 'BOOLEAN')
    {
        return $self->__GetHtmlBool($key);
    }
    elsif ($field_type eq 'BYTEA')
    {
        return $self->__GetHtmlBytea($key);
    }
    else
    {
        return "";
    }
}

=head2 Fonction __GetHtmlNumber

 Title    : __GetHtmlNumber
 Usage    : my $html=__GetHtmlNumber('table.champ')
 Fonction : Appele par GetHtml lorsque le type est REAL ou INTEGER - Ecrit un champ text de taille 10
            met la valeur par defaut en utilisant $o_param, si necessaire
 Args     : $key   La cle, exemple souche.numero (plus de field:)
 Retourne : le code html

=cut
sub __GetHtmlNumber
{
    my $self = shift;
    my ($key) = (@_);

    my $o_param = $self->{__o_param};
    my $value   = "";
    $value = $o_param->Get($key) if ($o_param->IsDefined($key));
    return "<input type=\"text\" name=\"$key\" size=\"10\" value=\"$value\" />";
}

=head2 Fonction __GetHtmlDate

 Title    : __GetHtmlDate
 Usage    : my $html=__GetHtmlDate('table.champ')
 Fonction : Appele par GetHtml lorsque le type est DATE
            met la valeur par defaut en utilisant $o_param, si necessaire
            Sinon ecrit la date d'aujourd'hui
 Args     : $key   La cle, exemple souche.date (plus de field:)
 Retourne : le code html

=cut
sub __GetHtmlDate
{
    my $self = shift;
    my ($key) = (@_);

    my $o_param = $self->{__o_param};
    my $value   = "";
    my $tmp_val='';
    if ($o_param->IsDefined($key) and $o_param->Get($key) != '')
    {
        $value = $o_param->Get($key);
    }

    my @now = localtime(time());
    my ($d,$m,$y) = ($now[3],$now[4],$now[5]);
    $tmp_val = sprintf("&nbsp;(ex. %02d-%02d-%04d)",$d,$m+1,$y+1900);
    return "<input type=\"text\" name=\"$key\" size=\"10\" value=\"$value\" />$tmp_val";    
}

=head2 Fonction __GetHtmlText

 Title    : __GetHtmlText
 Usage    : my $html=__GetHtmlText('table.champ')
 Fonction : Appele par GetHtml lorsque le type est TEXT
            Ecrit un champ de type textarea avec une taille codee en dur.
 Args     : $key   La cle, exemple souche.commentaire (plus de field:)
 Retourne : le code html

=cut
sub __GetHtmlText
{
    my $self = shift;
    my ($key) = (@_);

    my $o_param = $self->{__o_param};
    my $value   = "";
    $value = $o_param->Get($key) if ($o_param->IsDefined($key));
    return "<textarea name=\"$key\" rows=\"5\" cols=\"50\" >$value</textarea>";
}

=head2 Fonction __GetHtmlVarchar

 Title    : __GetHtmlVarchar
 Usage    : my $html=__GetHtmlvar('table.champ',10)
 Fonction : Appele par GetHtml lorsque le type est VARCHAR(10)
            Ecrit un champ de type input type=text avec une taille passee par parametres
 Args     : $key   La cle, exemple souche.commentaire (plus de field:)
            $size  La longueur du VARCHAR et du champ
 Retourne : le code html

=cut
sub __GetHtmlVarchar
{
    my $self = shift;
    my ($key,$size) = (@_);

    my $o_param = $self->{__o_param};
    my $value   = "";
    $value = $o_param->Get($key) if ($o_param->IsDefined($key));
    return "<input type=\"text\" name=\"$key\" size=\"$size\" value=\"$value\" />";
}

=head2 Fonction __GetHtmlBool

 Title    : __GetHtmlBool
 Usage    : my $html=__GetHtmlBool('table.champ')
 Fonction : Appele par GetHtml lorsque le type est bool - Ecrit un checkbox
 Args     : $key   La cle, exemple souche.check (plus de field:)
 Retourne : le code html

=cut
sub __GetHtmlBool
{
    my $self = shift;
    my ($key) = (@_);
    
    my $o_param = $self->{__o_param};
    my $value   = "";
    $value = $o_param->GetInteger($key) if ($o_param->IsDefined($key));
    my $checked = ($value==0) ? '' : 'checked="checked"';
    return "<input type=\"checkbox\"  name=\"$key\" value=\"1\" $checked \>";
}

=head2 Fonction __GetHtmlbytea

 Title    : __GetHtmlBytea
 Usage    : my $html=__GetHtmlBytea('table.champ')
 Fonction : Appele par GetHtml lorsque le type est bool - Ecrit un filefield
 Args     : $key   La cle, exemple souche.photo (plus de field:)
 Retourne : le code html

=cut
sub __GetHtmlBytea
{
    my $self = shift;
    my ($key) = (@_);
    
    my $o_cgi = $self->{__o_cgi};
    return $o_cgi->filefield(-name=>"$key");
}


=head2 Fonction __GetHtmlSelect,GetHtmlSelect,GetHtmlSelectGmi

 Title    : __GetHtmlselect
 Usage    : my $html = GetHtmlSelect('field:table.champ')
            my $html = GetHtmlSelect('field:table.champ','A..Z')
            my $html = GetHtmlSelectGmi('field:table.champ')
 Fonction : Doit etre utilisee via AnotherTemplate lorsqu'on a une table de jointure
            Recupere par &O2DBI::O2DBI_WebUtils::RetrieveFieldValuesById l'ensemble de toutes les valeurs se trouvant dans la table
            genere un element de type <select> afin de donner le choix entre toutes ces valeurs
            Les valeurs sont les id mais ce qui est affiche est le champ passe en parametre
            Ajoute une ligne blanche en tete de liste
 Args     : $key       La cle, exemple field:souche.nom (field: est obligatoire !)
            $ra_values ref vers un array ou string. Si string elle doit etre de la forme A..Z ou 1..9
 Retourne : le code html
 Globals  : Aucun

=cut
sub GetHtmlSelect
{
    my $self = shift;
    unshift(@_,0);
    return $self -> _GetHtmlSelect(@_);
}
sub GetHtmlSelectGmi
{
    my $self = shift;
    unshift(@_,1);
    return $self -> _GetHtmlSelect(@_);
}
sub _GetHtmlSelect
{
    my $self = shift;
    my ($gmi_flg,$key,$ra_values) = (@_);

    #warn join ('#',@_);
    my $html = "";
    my $ra_values_sorted;
    
    # If $ra_values defined and NOT a ref to an array, it should be X..Y: we build the array
    if (defined $ra_values)
    {
        if (ref $ra_values ne 'ARRAY')
        {
            my ($first,$last) = split ('\.\.',$ra_values);
            my @a_val = ($first..$last);
            $ra_values   = \@a_val;
        }
        $ra_values_sorted = &SortValues($ra_values);

        # Generate the html code
        $html  = "<select name=\"$key\">\n";
        $html .= "<option value=\"\" >&nbsp;</option>\n";
        foreach my $v (@$ra_values_sorted)
        {
            $html .= "<option value=\"$v\">$v</option>\n";
        }
        $html .= "</select>\n";
    }

    # $ra_values NOT defined = we look for the values found in the database
    else
    {
        # Parse the key name, discovering the table and the field
        my ($table,$field) = $self->__ParseKey($key);
        my %h_inv_values;
        my $rh_values = &O2DBI::O2DBI_WebUtils::RetrieveFieldValuesById($table,$field);
        $self->_Trace("RetrieveFieldValuesById called table=$table field=$field\n",'*',3);

        # Inverser le hash pour presenter les donnees triees par la suite
        foreach my $k (keys %$rh_values)
        {
            $h_inv_values{$rh_values->{$k}} = $k;
        }
        
        my @a_values = keys %h_inv_values;
        $ra_values   = \@a_values;
        my $ra_values_sorted = ($gmi_flg==0) ? &SortValues($ra_values) : &SortValuesGmi($ra_values);
        
        # Generate the html code
        $html = "<select name=\"$key\">\n";
        $html   .= "<option value=\"\" />\n";

        foreach my $v (@$ra_values_sorted)
        {
            # NOTE - We keep only the 1st 40 characters for display
            $html .= "<option value=\"$h_inv_values{$v}\">".substr($v,0,79)."</option>\n";
        }
        $html .= "</select>\n";
    }
    
    # sort the array passed by parameters alphanumeric, or numeric
    sub SortValues
    {
        my ($ra_values) = (@_);

        return $ra_values if(@$ra_values==0);

        my @a_values_sorted;
        
        # Alphanumeric sort
        if (grep (/\D/, @$ra_values) != 0)
        {
            @a_values_sorted = sort { lc($a) cmp lc($b) } @$ra_values;
            return \@a_values_sorted;

        }
        
        # numeric sort
        else
        {
            @a_values_sorted = sort { $a <=> $b } @$ra_values;
            return \@a_values_sorted;
        }
    }
    
    # sort the array passed by parameters convenient for codes (GMIxxx, pGMIxxx)
    sub SortValuesGmi
    {
        my ($ra_values) = (@_);

        return $ra_values if(@$ra_values==0);
        
        # chercher la premiere valeur non nulle
        my $first_value;
        foreach my $v (@$ra_values)
        {
            if ($v =~ /^\s*$/)
            {
                shift (@$ra_values);
                next;
            }
            $first_value = $v;
            last;
        }
        
        my @a_values_sorted;
        # Extraire le prefixe
        if ($first_value =~ /^([A-Z]+)\d+/i)
        {
            my $prefix = $1;
            my $l = length $prefix;
            
            # remove prefix, sort numerically, prepend prefix again
            my @a_num        = map (substr($_,$l), @$ra_values);
            my @a_num_sorted = sort { $a <=> $b } @a_num;
            @a_values_sorted = map $prefix.$_, @a_num_sorted;
            
            # return the sorted array
            return \@a_values_sorted;
        }
        else
        {
            return $ra_values;
        }
    }
}

=head2 Fonction GetHtmlAutocomplete

 Title    : GetHtmlAutocomplete
 Usage    : my $html = GetHtmlAutocomplete('field:table.champ')
            my $html = GetHtmlAutocomplete('field:table.champ','VARCHAR('10)')
 Fonction : Doit etre utilisee via AnotherTemplate
            Renvoit le code permettant de faire un YUI Autocomplete 
            Le champ appartiendra a la classe AutocompleteClass ou AutocompleteEditClass suivant les cas:
                     - si l'objet est en edit_mode ET SI le champ utilise pour l'autocompletion est le meme que le 
                     champ renseigne, ALORS on utilise la classe AutocompleteEditClass (c-a-d que l'utilisateur peut entrer une valeur differente des valeurs
                     deja existantes)
                     - si l'objet est en edit_mode ET si on utilise un autre champ pour l'autocompletion (cas des tables de jointure) ALORS
                     on utilise la classe AutocompleteClass (c-a-d que l'utilisateur est limite aux valeurs deja existantes).
                     - si l'objet n'est pas en edit_mode, on utilise la classe AutocompleteClass
 Args     : $key       La cle, exemple field:souche.nom (ces trois parties sont obligatoires)
            $field_type (optionnel) Le type de champ (probablement CHAR(nn) ou VARCHAR(nn)
                        On utilise $field_type pour savoir la taille du champ d'entree
                        Si $field_type n'est pas passe par parametre, on interroge le schema pour le connaitre
            $compl_key (optionnel) Le champ utilise pour l'autocompletion. Si non precise, on utilise $key
 Retourne : le code html
 Globals  : Aucun

=cut
sub GetHtmlAutocomplete
{
    my $self = shift;
    my ($key,$field_type,$compl_key) = (@_);
    if (!defined $field_type)
    {
        my ($table,$field) = $self->__ParseKey($key);
        $field_type = O2DBI::O2DBI_WebUtils::GetTableFieldType($self->{__rh_schema},$table,$field);
    }
    
    my ($alt_key) = ($key =~ /field:(.+)$/);
    if (!defined($compl_key))
    {
        $compl_key = $alt_key;
    }

    my ($size,$style) = ("","");
    if ($field_type =~ /(\d+)/)
    {
        $size = "size=\"$1\"";
        $style= "style=\"width:${1}em\"";
    }
    
    my $o_param = $self->{__o_param};
    my $value   = "";
    $value = $o_param->Get($key) if ($o_param->IsDefined($key));
    my $class = ($self->{__edit_mode} == 1 and $compl_key eq $alt_key) ? 'AutocompleteEditClass' : 'AutocompleteClass';
    return " <div class=\"$class\" id=\"ID_AC_$compl_key\" $style > <input type=\"text\" value=\"$value\" $size name=\"$key\" /> <div>&nbsp;</div></div>";
}

=head2 Fonction GetAutocompleteValues

 Title    : GetAutocomplete
 Usage    : my $ra_possible_values = GetHtmlAutocomplete('field:table.champ')
            my $ra_possible_values = GetHtmlAutocomplete('field:table.champ',\@a_values)
 Fonction : Doit etre utilisee via LipmAjaxDataClass (appele lorsqu'il y a un champ Autocomplete)
            Les differentes valeurs peuvent etre fournies par le second parametre, s'il n'est pas fourni on va les chercher dans la base
 Args     : $key       La cle, exemple field:souche.nom (field: est obligatoire !)
            $ra_values ref vers un array ou string. Si string on met des [] pour en faire un ref vers un array TODO (c'est pas encore fait !)
 Retourne : une ref vers un tableau contenant la liste triee de toutes les valeurs possibles
 Globals  : Aucun

=cut
sub GetAutocompleteValues
{
    my $self = shift;
    my ($key,$ra_values) = (@_);

    my ($table,$field) = $self->__ParseKey($key);
    if (!defined $ra_values)
    {
        my $rh_values = &O2DBI::O2DBI_WebUtils::RetrieveFieldValuesById($table,$field);
        $self->_Trace("RetrieveFieldValuesById called table=$table field=$field\n",'*',3);
        my @a_values = values %$rh_values;
        $ra_values   = \@a_values;
    }
        
    my @a_values_sorted;
    
    if (grep (/\D/, @$ra_values) or grep (/^$/, @$ra_values) != 0)
    {
        # il n'y a pas que des chiffres: tri non numerique !
        @a_values_sorted = sort { lc($a) cmp lc($b) } @$ra_values;
    }
    else
    {
        @a_values_sorted = sort { lc($a) <=> lc($b) } @$ra_values;
    }
    
    {
        my $prev = '';
        my @a_values_uniq;
        foreach my $v (@a_values_sorted)
        {
            if ($v ne $prev)
            {
                push(@a_values_uniq,$v);
                $prev=$v;
            }
        }
        @a_values_sorted = @a_values_uniq;
    }
    return \@a_values_sorted;
}

=head2 Fonction GetHtmlGMI

 Title    : GetHtmlGMI
 Usage    : my $html = GetHtmlGMI()
 Fonction : Doit etre utilisee via AnotherTemplate
            Ecrit un element de formulaire htlm, avec la valeur initialisee soit a un GMI correspondant au "prochain" numero disponible,
            soit a la valeur du parametre s'il existe
            Le prefixe est GMI par defaut, mais il est en fait dependant de l'utilisateur, il est donc lu dans la base.
 Args     : aucun
 Retourne : le code html
 Globals  : Aucun
 TODO - Cette fonction n'est pas du tout generique, elle est specifique a clism, ne devrait-elle pas etre ailleurs ?

=cut
sub GetHtmlGMI
{
    my $self = shift;
    my ($key) = (@_);
 
    my ($table,$field) = $self->__ParseKey($key);
    my $o_param        = $self->{'__o_param'};

    my $login = $o_param->Get('__wb_login');
    my $prefix = '';
    my $ra_o_utilisateur = &O2DBI::O2DBI_WebUtils::ExecuteFetchbySQL(" login LIKE '$login' ", "utilisateur");
    if (defined $ra_o_utilisateur->[0]->{prefixe} && ($ra_o_utilisateur->[0]->{prefixe} ne ''))
    {
        $prefix = $ra_o_utilisateur->[0]->{prefixe};
    }
    
    # Pas de prefixe du tout: on ne calcule rien
    else
    {
        return "<input type=\"text\" name=\"$key\" size=\"10\" value=\"\" />";
    }

    $prefix = "p$prefix" if ($table eq 'plasmide');
    
    my $value;
    if ($o_param->IsDefined($key))
    {
        $value = $o_param->Get($key);
    }
    else
    {
        my $ra_o_row = &O2DBI::O2DBI_WebUtils::ExecuteFetchbySQL(" code LIKE '$prefix" . "%'", $table);
        my $prochaincode = 0;
        foreach my $o_row (@{$ra_o_row})
        {
            my $code = $o_row->code;
            $code =~ s/^$prefix//;
            if ($code >= $prochaincode)
            {
                $prochaincode = $code;
            }
        }
        $prochaincode++;
        $value       = $prefix . $prochaincode;
    }
    return "<input type=\"text\" name=\"$key\" size=\"10\" value=\"$value\" />";
}

=head2 Fonction __ParseKey

 Title    : __ParseKey
 Usage    : my ($table,$champ) = __ParseKey('field:table.champ')
 Fonction : A partir d'une string formattee comme ci-dessus, renvoie un tableau comprenant $table et $champ
            Les differentes valeurs peuvent etre fournies par le second parametre, s'il n'est pas fourni on va les chercher dans la base
 Args     : $key       La cle, exemple field:souche.nom (field: est obligatoire !)
            $ra_values ref vers un array ou string. Si string on met des [] pour en faire un ref vers un array TODO (c'est pas encore fait !)
 Retourne : le code html
 Globals  : Aucun

=cut
sub __ParseKey
{
    my $self = shift;
    
    my ($key) = (@_);
    if ($key =~ /^field:(\w+)\.(\w+)$/)
    {
        return ($1,$2);
    }
    else
    {
        return ("","");   # should never happen
    }
}
1;

