# Copyright INRA/CNRS
# 
# Emmanuel.Courcelle@toulouse.inra.fr
# Sebastien.Carrere@toulouse.inra.fr
# 
# Ce logiciel est un programme informatique servant à [rappeler les
# caractéristiques techniques de votre logiciel]. 
# 
# Ce logiciel est régi par la licence [CeCILL|CeCILL-B|CeCILL-C] soumise au droit français et
# respectant les principes de diffusion des logiciels libres. Vous pouvez
# utiliser, modifier et/ou redistribuer ce programme sous les conditions
# de la licence [CeCILL|CeCILL-B|CeCILL-C] telle que diffusée par le CEA, le CNRS et l'INRIA 
# sur le site "http://www.cecill.info".
# 
# En contrepartie de l'accessibilité au code source et des droits de copie,
# de modification et de redistribution accordés par cette licence, il n'est
# offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
# seule une responsabilité restreinte pèse sur l'auteur du programme,  le
# titulaire des droits patrimoniaux et les concédants successifs.
# 
# A cet égard  l'attention de l'utilisateur est attirée sur les risques
# associés au chargement,  à l'utilisation,  à la modification et/ou au
# développement et à la reproduction du logiciel par l'utilisateur étant 
# donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
# manipuler et qui le réserve donc à des développeurs et des professionnels
# avertis possédant  des  connaissances  informatiques approfondies.  Les
# utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
# logiciel à leurs besoins dans des conditions permettant d'assurer la
# sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
# à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
# 
# Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
# pris connaissance de la licence [CeCILL|CeCILL-B|CeCILL-C], et que vous en avez accepté les
# termes.

package O2DBI::InsertForms;

use strict;
use ParamParser;
use O2DBI::O2DBI_WebUtils;
use CGI;
use Carp;
use Data::Dumper;
use Error qw( :try );
use LipmError;

=head2 procedure PrintForm

 Title		  : PrintForm
 Usage		  : $ra_html_lines = &PrintForm($o_cgi, $rh_param)
 Prerequisite 	  : none
 Function	  : print the form in order to add new records in the database
 Returns	  : $ra_html_lines : HTML lines 
 Args		  : none
 Globals	  : none

=cut

sub PrintForm
{

    my ($rh_param, $o_cgi, $table, $ra_fields_to_hidde) = (@_);

    my $rh_schema = &O2DBI::O2DBI_WebUtils::GetSchemaStructure();
    my ($ra_champs, $rh_field_value) = &O2DBI::O2DBI_WebUtils::RetrieveFieldAndValue($table, $rh_schema);
    my $rh_fields_type = &O2DBI::O2DBI_WebUtils::RetrieveFieldsType($table, $rh_schema);

    my $ra_html_lines = &OrganizeForm($ra_champs, $rh_fields_type, $o_cgi, $table, $ra_fields_to_hidde, $rh_schema);

    return ($ra_html_lines);

}

=head2 procedure OrganizeForm

 Title		  : OrganizeForm
 Usage		  : &OrganizeForm($rh_champs, $rh_fields_type, $o_cgi, $table, $ra_fields_to_hidde, $rh_schema)
 Prerequisite     : none
 Function	  : Foreach field in $rh_champs, this function arrange these elements in an array of 2 columns
 Returns	  : a reference to an array that correspond to all the lines of the final array
 Args		  : $rh_champs: is a reference to an array of fields of 'ORF'
                    $o_cgi: object that refrences CGI
 Globals	  : none

=cut

sub OrganizeForm
{
    my ($ra_champs, $rh_fields_type, $o_cgi, $table, $ra_fields_to_hidde, $rh_schema) = (@_);
    #### le tableau sera organise sur 2 COLONNES

    my @a_sorted_field   = ();
    my @a_td_col         = ();
    my @a_tr_line        = ();
    my $color_main_field = "#BBBBBB";

    #je donne l'ordre d'affichage des donnees, avec un element en plus
    @a_sorted_field = (sort @{$ra_champs});

    for (my $i = 0; $i <= $#a_sorted_field; $i++)
    {
        if (grep (/^$a_sorted_field[$i]$/, @{$ra_fields_to_hidde}) < 1)
        {
            my ($align,   $align_2)   = ('left',    'left');
            my ($valign,  $valign_2)  = ('middle',  'middle');
            my ($bgcolor, $bgcolor_2) = ("#DDDDDD", "#DDDDDD");
            my ($colspan, $colspan_2) = '';
            my $data   = $a_sorted_field[$i];
            my $data_2 = '';

            if ($a_sorted_field[$i] =~ /_id$/)
            {
                my $linked_table = $a_sorted_field[$i];
                $linked_table =~ s/_id$//;
                my $main_field       = &O2DBI::O2DBI_WebUtils::RetrieveMainField($linked_table,       $rh_schema);
                my $rh_liste_valeurs = &O2DBI::O2DBI_WebUtils::RetrieveFieldValuesById($linked_table, $main_field);

                $data_2 =
                  $o_cgi->popup_menu(
                                     -name    => $a_sorted_field[$i],
                                     -values  => [keys %{$rh_liste_valeurs}],
                                     -labels  => $rh_liste_valeurs,
                                     -default => ''
                                     );

            }
            else
            {
                my $field_type = $rh_fields_type->{$a_sorted_field[$i]};
                $data_2 = &InsertWrittingArea($a_sorted_field[$i], $field_type, $o_cgi, $table, $rh_schema);

            }
            $align_2 = 'right';
            push(@a_td_col,
                 $o_cgi->td({-align => $align, -valign => $valign, -bgcolor => $bgcolor, -colspan => $colspan}, $data),
                 "\n");

            if ($data_2 ne '')
            {
                push(
                     @a_td_col,
                     $o_cgi->td(
                                {
                                 -align   => $align_2,
                                 -valign  => $valign_2,
                                 -bgcolor => $bgcolor_2,
                                 -colspan => $colspan_2
                                },
                                $data_2
                       ),
                     "\n"
                     );
            }

            push(@a_tr_line, $o_cgi->Tr({-align => 'LEFT', -valign => 'MIDDLE'}, @a_td_col), "\n");
            @a_td_col = ();
        }

    }

    return (\@a_tr_line);
}

=head2 procedure InsertWrittingArea

 Title		  : InsertWrittingArea
 Usage		  : &InsertWrittingArea($field, $o_cgi)
 Prerequisite     : none
 Function	  : depending on $field, it put popup_menu, textfield, or textArea in a variable
 Returns	  : a variable that contains data that will be put into tha array
 Args		  : $field: field
                    $o_cgi: object that refrences CGI
		    $table
		    $rh_schema
 Globals	  : none

=cut

sub InsertWrittingArea
{

    my ($field, $field_type, $o_cgi, $table, $rh_schema) = (@_);

    #recuperation du type
    my $max_length = &O2DBI::O2DBI_WebUtils::ReturnMaxLengthData($table, $field, $rh_schema);

    my $length         = 25;
    my $valeur_cellule = '';

    if ($field =~ /^upload$/)
    {
        $valeur_cellule =
          $o_cgi->filefield(
                            -name      => $field,
                            -default   => '',
                            -size      => 50,
                            -maxlength => 80
                            );
    }
    elsif (($field_type eq 'INTEGER') || ($field_type eq 'DATE') || ($field_type =~ /VARCHAR\((\d+)\)/))
    {
        $valeur_cellule =
          $o_cgi->textfield(
                            {
                             -name      => $field,
                             -default   => '',
                             -size      => $length,
                             -maxlength => $max_length
                            }
                            );
    }
    elsif ($field_type eq 'TEXT')
    {
        $valeur_cellule =
          $o_cgi->textarea(
                           -name    => $field,
                           -default => '',
                           -rows    => '5',
                           -columns => $length
                           );

    }
    elsif ($field_type eq 'BOOL')
    {
        $valeur_cellule =
          $o_cgi->radio_group(
                              -name    => $field,
                              -values  => ['oui', 'non'],
                              -default => ''
                              );

    }

    elsif ($field_type =~ /ENUM\((.+)\)/)
    {
        my @a_enum_values = split(/,/, $1);
        my %h_popup_values = ();

        foreach my $enum_value (@a_enum_values)
        {
            $enum_value =~ s/^'//;
            $enum_value =~ s/'$//;
            $h_popup_values{$enum_value} = $enum_value;
        }

        $valeur_cellule =
          $o_cgi->popup_menu(
                             -name    => $field,
                             -values  => [keys %h_popup_values],
                             -labels  => \%h_popup_values,
                             -default => ''
                             );

    }

    else
    {
        print STDERR "$field ne correspond a aucun champ\n";
    }

    return ($valeur_cellule);

}

=head2 function InsertNewElement

 Title		  : InsertNewElement
 Usage		  : $boolean = &InsertNewElement($rh_param, $table)
 Function	  : Check data compatibility, then insert a new element in the database
                ...OR delete data from the database
 Return       : The id of the created object
 On error     : throw a LipmError
 Args		  : $o_param: ParamParser structure
                $table: table to deal with
 Globals	  : none

=cut

sub InsertNewElement
{
    my ($o_param, $table) = (@_);
    croak "ERREUR PARAMETRES" if ($#_ != 1);

    #RECUPERATION DES CHAMPS + VERIFICATION DES TYPES

    my %h_fields_value = ();
    my %h_fields_del_value = ();
   
    my $rh_schema = &O2DBI::O2DBI_WebUtils::GetSchemaStructure();
    my $ra_fields = &O2DBI::O2DBI_WebUtils::RetrieveFields($table, $rh_schema);
    #    print "<h2>Insertion // Mise a jour</h2>";
    
    # 'id' est toujours present, il peut etre utilise pour supprimer un element
    &ValuesToDel($o_param,\%h_fields_del_value,$table,'id');
    
    # Boucler sur tous les champs de la table
    foreach my $field (@{$ra_fields})
    {
        # On saute l'element 'id', car il est deja traite
        if ($field eq 'id')
        {
            next;
        }
        else
        {
            # Signale une table de jointure, ex: souche.espece_id
            if ($field =~ /_id$/)
            {
                # $joined_table = espece
                my ($joined_table) = ($field =~ /(\w+)_id$/);
                
                # La liste des champs field:espece.champ, on recupere field:espece.nom
                my @a_field_name = $o_param->GetKeys('field:' . $joined_table . '\.\w+');
                
                # Si plusieurs: ERREUR !
                if ($#a_field_name > 0)
                {
                    carp("InsertNewElement, table $table failed: Error in the jointure $joined_table");
                    throw LipmError (-text => "Error in the jointure $joined_table");
                }
                
                # Si un seul, c'est OK - On le stocke dans $h_fields_value{'element_id'}
                elsif ($#a_field_name == 0)
                {
                    my $value = $o_param->Get($a_field_name[0]);
                    $h_fields_value{$field} = $value;                    
                }
                
                # Si aucun, je ne comprends pas (pas de field:id a l'horizon, on met '' dans tous les cas !)
                else
                {
                    my $value = $o_param->Get('field:id');
                    $h_fields_value{$field} = $value;
                }
                
                # Rechercher des demandes de suppression correspondant a ce champ
                &ValuesToDel($o_param,\%h_fields_del_value,$joined_table,$field);
            }
        
            #Signale un champ ordinaire, ex code
            else
            {
                my $value = '';
                
                if ($o_param->IsDefined('field:' . $table . '.' . $field))
                {
                    $value = $o_param->Get('field:' . $table . '.' . $field);
                }
                
                if ($value ne '')
                {
                    warn "field:$table.$field#$value#";

                    $value = '' if ($value eq '-');
                    
                    my $correspondance =
                    &O2DBI::O2DBI_WebUtils::CheckDataCompatibility($rh_schema, $table, $field, $value);
                    
                    # ERROR !
                    if ($correspondance eq '0')
                    {
                        carp("InsertNewElement, table $table failed: Error in the field $field: $value not accepted");
                        throw LipmError (-text => "Error in the field $field: $value not accepted");
                    }
                    elsif ($correspondance == 2)
                    {
                        $value = &UploadFileContent($value);
                    }
                    elsif ($correspondance ne '1')  #CAS DES REAL pour substituer les "," par des "." et des dates
                    {
                        $value = $correspondance;
                    }
                    $h_fields_value{$field} = $value;
                }
            }
        }
    }

    # Suppression des donnees
    if (keys %h_fields_del_value != 0)
    {
        &DeleteObject($o_param, \%h_fields_value,\%h_fields_del_value, $table, $rh_schema);

        # On avait a priori des donnes a supprimer: dans ce cas on masque les erreurs qui seront probablement generees si on n'a rien entre 
        # TODO ca n'est pas fameux car peut-etre que l'erreur masquee etait en fait importante ?
        try
        {
            my $id = &CreateObject($o_param, \%h_fields_value, $table, $rh_schema);
            return $id;
        }
        catch LipmError with
        {
            return '';
        };
    }
    else
    {
        # Insertion des donnees dans la database - Retourne le id
        return (&CreateObject($o_param, \%h_fields_value, $table, $rh_schema))
    }
    
    # Ajoute si necessaire une entree a %h_fields_del_value
    sub ValuesToDel
    {
        my ($o_param,$rh_fields_del_value,$table,$field) = (@_);
        
        # Etablir la liste des champs correspondant a des demandes de suppression
        my @a_field_del_name = $o_param->GetKeys('fieldD\d+:' . $table . '\.\w+');
        #warn ("koukou $table $field ".Dumper \@a_field_del_name);
        
        if (@a_field_del_name != 0)
        {
            my @a_values= ();
            foreach my $f (@a_field_del_name)
            {
                my $value = $o_param->Get($f);
                push (@a_values,$value);
            }
            $rh_fields_del_value->{$field} = \@a_values;
        }
    }
}


=head2 Function DeleteObject

 Title     : DeleteObject
 Usage	   : $boolean = &DeleteObject($o_param,$rh_fields_value,$rh_fields_del_value,$table)
 Procedure : Create a new O2DBI object to be deleted from the database
             Use this object to delete data
 Args      : $o_param: ParamParser structure
		     $rh_fields_value: donnees (cle = table.champ)
             $rh_fields_del_value: donnees a supprimer (cle = table.champ)
		     $table: table to deal with
             $rh_schema: the schema
 Return    : The id of the created object
 On error  : throw a LipmError
 Globals   : $O_CFG, $O_LOGGER

=cut

sub DeleteObject
{
    my ($o_param, $rh_fields_value, $rh_fields_del_value, $table, $rh_schema) = (@_);
    croak "ERREUR PARAMETRES" if ($#_ != 4);
    
    my $o_logger= $main::O_LOGGER;
    my $o_cfg   = $main::O_CFG;
    $o_logger->Trace("Entering DeleteObject table = $table\n",'*','3');
    my %h_fields_value = %$rh_fields_value;
    
    my $module   = &O2DBI::O2DBI_WebUtils::UseModule($table);
    my $o_object = '';
    
    # We retrieve the constructor fields from the schema
    my $ra_ra_constructor_fields = &O2DBI::O2DBI_WebUtils::RetrieveConstructorFields($table, $rh_schema);
    
    # Foreach possible constructors...
    foreach my $ra_constructor_fields (@$ra_ra_constructor_fields)
    {
        # For each possible constructor
        foreach my $ra_constructor_fields (@$ra_ra_constructor_fields)
        {
            # Marking we have deleted something at this iteration
            my $deleted_flg = 0;

            # Loop for every element in $rh_fields_del_value
            foreach my $f (keys %$rh_fields_del_value)
            {
                # Each element in $rh_fields_del_value is an array of ids to delete !
                my $ra_values = $rh_fields_del_value->{$f};
                foreach my $g (@$ra_values)
                {
                    $h_fields_value{$f} = $g;
                    my @a_constructor_parameters = ();
                    my $constructor_field_not_found = 0;
                    foreach my $constructor_field (@{$ra_constructor_fields})
                    {
                        if (!defined($h_fields_value{$constructor_field}) or ($h_fields_value{$constructor_field} eq ''))
                        {
                            $constructor_field_not_found = 1;
                            last;
                        }
                        else
                        {
                            #warn ("TABLE $table $creator_field => $rh_fields_value->{$creator_field}");
                            push(@a_constructor_parameters, $h_fields_value{$constructor_field});
                        }
                    };
                    
                    last if ($constructor_field_not_found);
                    
                    my $init_fct = $module.'->init_'.join('_',@$ra_constructor_fields).' ('.join(',',@a_constructor_parameters).')' ;
                    #warn 'KOUKOU '.$init_fct;
                    $o_object = eval "$init_fct";
                    if ($o_object == -1)
                    {
                        my $msg = "table $table failed: ".join(",",@$ra_constructor_fields).' => '.join(",",@a_constructor_parameters).")";
                        carp "DeleteObject, $msg";
                        throw LipmError (-text => "Delete data in $msg");
                    }   
                    
                    # The object was successfully created, we delete it now (AND the database data)
                    else
                    {
                        $deleted_flg=1;
                        $o_object->delete();
                        
                        # Try to create the same object again: it is succeeds, the object could not be deleted
                        my $o_n_object = eval "$init_fct";
                        if ($o_n_object != -1)
                        {
                            my $msg = "table $table failed: could not delete object (probably used in some table)";
                            carp "DeleteObject, $msg";
                            throw LipmError (-text => "Delete data in $msg");
                        }
                    }
                }
                
                # We delete the field from $rh_fields_del_value, avoiding a second (failed) deletion at next iteration
                delete $rh_fields_del_value->{$f} if ($deleted_flg);
            }
        }
    }
}

=head2 Function CreateObject

 Title		  : CreateObject
 Usage		  : my $id = &CreateObject($o_param, $rh_fields_value, $table, $rh_schema)
 Prerequisite : fields corresponding to uploaded files must be named 'upload'
 Function	  : Create a new O2DBI object to be inserted in databas, OR update an already existing object
                Use this object to update the database
 Args		  : $o_param: ParamParser structure
		        $rh_fields_value: donnees a inserer (cle = table.champ)
		        $table: table to deal with
                $rh_schema: the schema
 Return       : The id of the created object
 On error     : throw a LipmError
 Globals	  : $O_CFG, $O_LOGGER

=cut

sub CreateObject
{
    my ($o_param, $rh_fields_value, $table, $rh_schema) = (@_);
    croak "ERREUR PARAMETRES" if ($#_ != 3);
    
    my $o_logger= $main::O_LOGGER;
    my $o_cfg   = $main::O_CFG;
    $o_logger->Trace("Entering CreateObject table = $table\n",'*','3');
    
    my $module   = &O2DBI::O2DBI_WebUtils::UseModule($table);
    my $o_object = '';

    # If .id defined, we update an existing record
    #if (($o_param->IsDefined('field:id')) && !($o_param->IsDefined('table')))
    if ($o_param->IsDefined("field:${table}.id"))
    {
        my $id = $o_param->Get("field:${table}.id");
        $o_object = $module->init_id($id);
        $o_logger->Trace("Appel de init_id pour table $table id $id\n",'*','3');
    }
    
    # else, we create a new object and we keep track of the id in $o_param
    else
    {
        my $ra_creator_fields = &O2DBI::O2DBI_WebUtils::RetrieveCreatorFields($table, $rh_schema);
        my @a_creator_parameters = ();
        #$o_logger->Trace(join (" ",@{$ra_creator_fields})."\n","",'4');
        foreach my $creator_field (@{$ra_creator_fields})
        {
            next if ($creator_field eq 'id');
            if ((($rh_fields_value->{$creator_field} eq '') || (!defined($rh_fields_value->{$creator_field}))))
            {
                carp("CreateObject, table $table failed: creator $creator_field undefined (creators= ".join (" ",@{$ra_creator_fields}).")");
                throw LipmError (-text => "Sorry, the field $creator_field is required");
            }
            
            if ($creator_field eq 'upload')
            {
                my $upload_dir = $o_cfg->Get('ROOT') . $o_cfg->Get('UPLOAD_DIR');
                my $upload_file = &UploadFile($o_param, $creator_field, $upload_dir);
                $rh_fields_value->{$creator_field} =
                $o_cfg->Get('SERVERNAME')
                . $o_cfg->Get('HTTP_ROOT')
                . $o_cfg->Get('UPLOAD_DIR') . "/"
                . $upload_file;
            }
            
            #warn ("TABLE $table $creator_field => $rh_fields_value->{$creator_field}");
            push(@a_creator_parameters, $rh_fields_value->{$creator_field});
            
            #si je laisse ca , l'update marche pas pour les especes sur les souches ...
            #delete $rh_fields_value->{$creator_field};
        }

        $o_object = $module->create(@a_creator_parameters);
        if ($o_object == -1)
        {
            my $msg = "table $table failed: ".join(",",@$ra_creator_fields).' => '.join(",",@a_creator_parameters).")";
            carp "CreateObject, $msg";
            throw LipmError (-text => "Insert new data in $msg");
        }   
        
        my $id;
        $id = $o_object->id();
        $o_param->Set("field:${table}.id",$id);
        $o_logger->Trace("Appel de create pour table $table ==> id $id\n",'*','3');
        $o_logger->Trace(Dumper(@$ra_creator_fields)."\n",'','4');
        $o_logger->Trace(Dumper(@a_creator_parameters)."\n",'','4');
    }
    
    # Now, we update the database with the opional fields
    return (&UpdateTable($table, $o_object->{'id'}, $rh_fields_value));
}

=head2 function UpdateTable

 Title		  : UpdateTable
 Usage		  : $boolean = &UpdateTable($table, $id, $rh_new_field_value)
 Prerequisite 	  : none
 Function	  : Update an existing database record
 Return       : The id of the updated object
 On error     : throw a LipmError
 Args		  : $table: table to deal with
		        $id: record id to update
		        $rh_new_field_value: new record data
 Globals	  : none

=cut

sub UpdateTable
{
    my ($table, $id, $rh_new_field_value) = (@_);
    
    my $module   = &O2DBI::O2DBI_WebUtils::UseModule($table);
    my $o_object = $module->init_id($id);
    
    if ($o_object == -1)
    {
        my $msg = "Update data in table $table failed: ".join(",",keys %$rh_new_field_value).' => '.join(",",values %$rh_new_field_value);
        carp "UpdateTable, $msg";
        throw LipmError (-text => "$msg");
    }
    else
    {
        my %h_champ_val = ();
        foreach my $key (keys %{$rh_new_field_value})
        {
            #print STDOUT "$key " . $o_object->$key . " < $rh_new_field_value->{$key} > <br>";
            #if (($o_object->$key ne $rh_new_field_value->{$key}) && ($rh_new_field_value->{$key} ne ''))
            if (($o_object->$key ne $rh_new_field_value->{$key}))
            {
                $o_object->$key($rh_new_field_value->{$key});
            }
            
        }
    }
    return $id;
}

=head2 function UploadFile

 Title		  : UploadFile
 Usage		  : $file_name = &UploadFile($rh_param, $html_field_name, $upload_dir)
 Prerequisite 	  : none
 Function	  : Upload a file
 Returns	  : $file_name
 Args		  : $rh_param: ParamParser structure
 		    $html_field_name: html input filefield name
		    $upload_dir : target directory 
 Globals	  : none

=cut

sub UploadFile
{
    my ($rh_param, $html_field_name, $upload_dir) = @_;

    my $file = $rh_param->Get($html_field_name);

    my $new_uploaded_file_name = $file;

    my $file_name_for_linux = $file;
    $file_name_for_linux =~ s/\s/_/g;
    my $new_uploaded_file_name = $file_name_for_linux;

    my $counter = 0;
    while (-e "$upload_dir/$new_uploaded_file_name")
    {
        $counter++;
        $new_uploaded_file_name = $counter . "_" . $file_name_for_linux;
    }

    open(OUT, "> $upload_dir/$new_uploaded_file_name");
    binmode $file;
    while (my $line = <$file>)
    {
        print OUT $line;
    }

    close OUT;

    return $file;

}

sub UploadFileContent
{
    my ($file) = @_;
    my $file_content = '';
	print STDOUT "<b>Insere un fichier $file</b><br>";

    binmode $file;
    while (my $line = <$file>)
    {
	
        $file_content .= $line;
    }

    use MIME::Base64;
    $file_content = MIME::Base64::encode($file_content);

    return $file_content;

}

1;
