package O2DBI::O2DBI_WebUtils;

use strict;

#use ParamParser;
#use CGI;
use Digest::MD5 qw(md5 md5_hex md5_base64);
use O2DBI::InsertForms;
use O2DBI::DisplayInputField;
use clism::DBMS;
use Carp;

    
use SchemaDB qw( $DB_NAME $COOKIE_PRIVILEGE_PARAM_NAME $COOKIE_USERID_PARAM_NAME %H_SCHEMA_DB);

#########################################################################################

=head2 fonction GetTemplate

 Title		: GetTemplate
 Usage		: my $path = GetTemplatePath('some_template');
 Function   : Construit un template a partir du nom passse en parametres
              Le chemin du fichier de template depend de %CFG et du parametre __wb_lang
 Returns    : Le template construit
 Args       : $template Le nom de fichier (.inc)
 Globals    : $O_CFG, $O_PARAM

=cut

sub GetTemplate
{
    my ($template) = (@_);
    my $path = $main::O_CFG->Get('ROOT') . '/' .$main::O_CFG->Get('TEMPLATE_DIR') . '/' . $main::O_PARAM->Get('__wb_lang') . '/' . $main::O_CFG->Get($template);
    croak "ERROR - Cannot find $path for $template" if (! -f $path);
    return New AnotherTemplate($path);
}

=head2 procedure GetSchemaStructure

 Title		: GetSchemaStructure
 Usage		: $rh_schema = &GetSchemaStructure()
 Prerequisite : none
 Function	 : returns the structure of the database (format O2DBI+) 
 Returns	 : a reference to an hash
 Args		 : none
 Globals	 : none

=cut

sub GetSchemaStructure
{
    my %h_schema = %H_SCHEMA_DB;
    return (\%h_schema);
}

=head2 procedure RetrieveFields

 Title		  : RetrieveFields
 Usage		  : &RetrieveFields($objet)
 Prerequisite : none
 Function	  : make an array that contains fields of a table
 Returns	  : a reference to this array
 Args		  : $table: table that you want to retrieve fields
                    $rh_schema: O2DBI+ schema
 Globals	  : none

=cut

sub RetrieveFields
{
    my ($table, $rh_schema) = (@_);

    my @a_fields = ();

    if (defined $rh_schema->{$table})
    {
        @a_fields = keys %{$rh_schema->{$table}->{'members'}};
    }

    return (\@a_fields);
}

=head2 procedure RetrieveCreatorFields

 Title		  : RetrieveCreatorFields
 Usage		  : $ra_mandatory_fields = &RetrieveCreatorFields($table)
 Prerequisite     : none
 Function	  : make an array of arrays containing fields of an object mandatory for the creator
 Returns	  : a reference to this array
 Args		  : $table: table that you want to retrieve mandatory fields
                    $rh_schema: O2DBI+ schema
 Globals	  : none

=cut

sub RetrieveCreatorFields
{
    my ($table, $rh_schema) = (@_);
    if (defined $rh_schema->{$table})
    {
        return $rh_schema->{$table}->{'creator'};
    }
    return;
}

=head2 procedure RetrieveConstructorFields

 Title		  : RetrieveConstructorFields
 Usage		  : $ra_fields = &RetrieveConstructorFields($table)
 Function	  : make an array that contains fields of an object mandatory for constructor
 Returns	  : a reference to this array
 Args		  : $table: table that you want to retrieve mandatory fields
                $rh_schema: O2DBI+ schema
 Globals	  : none

=cut
sub RetrieveConstructorFields
{
    my ($table, $rh_schema) = (@_);
    my @rvl=();
    if (defined $rh_schema->{$table})
    {
        push (@rvl,['id']);
        push (@rvl,@{$rh_schema->{$table}->{'constructors'}});
    }
    return \@rvl;
}

=head2 procedure RetrieveMainField

 Title		  : RetrieveMainField
 Usage		  : $main_field = &RetrieveMainField($table)
 Prerequisite : none
 Function	  : return the reference field of a table (displayed for cross reference tables)
 Returns	  : main field (a UNIQUE field in your DB)
 Args		  : $table: table that you want to retrieve fields
                    $rh_schema: O2DBI+ schema
 Globals	  : none

=cut

sub RetrieveMainField
{
    my ($table, $rh_schema) = (@_);

    if (defined $rh_schema->{$table}->{'main'})
    {
        return $rh_schema->{$table}->{'main'};
    }
    return;
}

=head2 procedure RetrieveFieldsType

 Title		  : RetrieveFieldsType
 Usage		  : $rh_fields_type = &RetrieveFieldsType($table)
 Prerequisite : none
 Function	  : make an hash that contains fields of a table with associated SQL type
 Returns	  : a reference to this hash
 Args		  : $table: table that you want to retrieve fields
                    $rh_schema: O2DBI+ schema
 Globals	  : none
 TODO A_JETER

=cut

sub RetrieveFieldsType
{
    my ($table, $rh_schema) = (@_);

    my %h_fields_type = ();

    if (defined $rh_schema->{$table})
    {
        foreach my $fields (keys %{$rh_schema->{$table}->{'members'}})
        {
            $h_fields_type{$fields} = $rh_schema->{$table}->{'members'}->{$fields};
        }
    }
    return (\%h_fields_type);
}

=head2 Function GetTableFieldType

 Title		  : GetTableFieldType
 Usage		  : my $type = GetTableFieldType($rh_schema,$table,$field)
 Function	  : Retrieve the db type from a (schema, table, field) triplet
 Args		  : $rh_schema: The schema to read
                $table    : the table
                $field    : the field
 Returns	  : The type
 Globals	  : none
 TODO: Un objet

=cut
sub GetTableFieldType
{
    my ($rh_schema,$table,$field) = (@_);
    return 'INTEGER' if ($field eq 'id');
    if (exists $rh_schema->{$table})
    {
        if (exists $rh_schema->{$table}->{members}->{$field})
        {
            return $rh_schema->{$table}->{members}->{$field};
        }
        else
        {
            croak ("ERROR - Cannot find field $field from table $table");
        }
    }
    else
    {
        croak ("ERROR - Cannot find table $table");
    }
}

=head2 procedure ExecuteFetchall

 Title		  : ExecuteFetchall
 Usage		  : &ExecuteFetchall($nom_objet)
 Prerequisite : none
 Function	  : call the fetchall fonction according to the object
 Returns	  : a reference to this array
 Args		  : $object: object that you want to know all the data
 Globals	  : none

=cut

sub ExecuteFetchall
{
    my ($object) = (@_);
    my $module = &UseModule($object);
    return ($module->fetchall());
}

=head2 procedure ExecuteFetchbySQL

 Title		: ExecuteFetchbySQL
 Usage		: $ra_fetchbysql = &ExecuteFetchbySQL($statement, $object)
 Function	: call the fetchall fonction according to the statement and the object
 Returns	: a reference to this array
 Args		: $statement: the condition in order to complete the SQL command
 			  $object: object that you want to know all the data
 Globals	  : none

=cut

sub ExecuteFetchbySQL
{
    my ($statement, $object) = (@_);
    my $module        = &UseModule($object);
    my $ra_fetchbysql = $module->fetchbySQL($statement);
    return ($ra_fetchbysql);
}

=head2 procedure ExecuteFetchbyID

 Title		: ExecuteFetchbyID
 Usage		: $rh_fetchbyid = &ExecuteFetchbyID($object)
 Prerequisite : none
 Function	 : it calls the right "fetchallby_id"
 Returns	  : a reference to an hash that contains all data of $nom_objet order by id field
 Args		 : $object: object concerned
 Globals	  : none

=cut

sub ExecuteFetchbyID
{
    my ($object)     = (@_);
    my $module       = &UseModule($object);
    my $rh_fetchbyid = $module->fetchallby_id();
    return ($rh_fetchbyid);
}

=head2 procedure CheckDataCompatibility

 Title	   	  : CheckDataCompatibility
 Usage		  : &CheckDataCompatibility($rh_schema, $table, $champs, $valeur)
 Prerequisite : none
 Function	  : checks if the data entered ($valeur) in a field is compatible with the same field in the database
 Returns	  : 0 (false),  2 (il y a un fichier a uploader), ou autre chose (dans ce cas c'est $valeur corrige (cf REAL))
 TODO         : refaire le prototype de cette ignoble fonction (valeur de retour bizarre)
 Args		  : $rh_schema: The schema
 				$table: the table
				$champs: the field
				$valeur: data to check
 Globals	  : none

=cut

sub CheckDataCompatibility
{
    my ($rh_schema, $table, $champs, $valeur) = (@_);
    
    my $correspond = '0';
    my $type_sql = $rh_schema->{$table}->{'members'}->{$champs};
    
    if (($type_sql eq 'INTEGER') and ($valeur =~ /\d+/))
    {
        $correspond = '1';
    }
    elsif ($type_sql =~ /^VARCHAR\((\d+)\)/)
    {
        if (length($valeur) <= $1)
        {
            $correspond = '1';
        }
    }
    elsif ($type_sql =~ /^CHAR\((\d+)\)/)
    {
        if (length($valeur) == $1)
        {
            $correspond = '1';
        }
    }
    elsif ($type_sql =~ /^CHAR$/)
    {
        if (length($valeur) == 1)
        {
            $correspond = '1';
        }
    }
    
    elsif ($type_sql eq 'DATE')
    {
        if (($valeur =~ /\d{4}-(\d{2})-(\d{2})/) and ($1 >= 1) and ($1 <= 12) and ($2 >= 1) and ($2 <= 31))
        {
            $correspond = '1';
        }
        elsif (($valeur =~ /(\d{2})-(\d{2})-(\d{4})/) and ($1 >= 1) and ($1 <= 31) and ($2 >= 1) and ($2 <= 12))
        {
            $correspond = "$3-$2-$1";
        }
    }
    elsif ($type_sql eq 'TEXT')
    {
        $correspond = '1';
    }
    elsif (($type_sql =~ /BOOL/i) and ($valeur =~ /[0,1]/))
    {
        $correspond = '1';
    }
    elsif ($type_sql eq 'REAL')
    {
    	if ($valeur =~ /,/)
    	{
            $valeur =~ s/,/\./g;
            #	print STDOUT "<h1>TEST OK</h1>";
        }
        
        if ($valeur =~ /\d+\.?\d*/)
    	{
        	$correspond = $valeur;
    	}
        
    }
    elsif ($type_sql =~ /ENUM\((.+)\)/)
    {
        my $values_string = $1;
        $values_string =~ s/','/,/g;
        $values_string =~ s/^'//;
        $values_string =~ s/'$//;
        
        my @a_enum_values = split(/,/, $values_string);
        
        $correspond = grep (/^$valeur$/, @a_enum_values);
    }
    elsif ($type_sql =~ /BYTEA/)
    {
    	$correspond = '2';
    }
    return ($correspond);
}

=head2 procedure UseModule

 Title		  : UseModule
 Usage		  : &UseModule($nom_objet, $extension)
 Prerequisite : none
 Function	  : use the right module
 Returns	  : the module that corresponds to the right table ($nom_objet)
 Args		  : $object: table of the data base
 		        $extension: argument that can contains "add" if $nom_objet_add is requested
 Globals	  : none

=cut

sub UseModule
{
    my ($object, $extension) = (@_);
    my $module = '';

    if ((!defined $object) or ($object eq ''))
    {
        confess "ERROR - UseModule called with empty object";
    }
    
    if ((defined $extension) && ($extension ne ''))
    {
        $module = $DB_NAME . "::" . $object . "_" . $extension;
        eval("use $module");
        warn $@ if $@;
    }
    else
    {
        $module = $DB_NAME . "::" . $object;
        eval("use $module");
        warn $@ if $@;
    }
    return ($module);
}

=head2 procedure ReturnMaxLengthData

 Title		  : ReturnMaxLengthData
 Usage		  : &ReturnMaxLengthData($table, $field, $rh_schema)
 Prerequisite     : GetSchemaStructure
 Function	  : search the right type of $champ
 Returns	  : the length of the type of $champ
 Args		  : $table: table of the data base
 		    $field: field of the table
		    $rh_schema
 Globals	  : none

=cut

sub ReturnMaxLengthData
{
    my ($table, $field, $rh_schema) = (@_);

    my $type   = $rh_schema->{$table}->{'members'}->{$field};
    my $length = '';

    if ($type eq 'INTEGER')
    {
        $length = 10;
    }
    elsif ($type eq 'REAL')
    {
        $length = 10;
    }
    elsif ($type =~ /VARCHAR\((\d+)\)/)
    {
        $length = $1;
    }
    elsif ($type =~ /CHAR\((\d+)\)/)
    {
        $length = $1;
    }
    elsif ($type eq 'TEXT')
    {
        $length = 1000;
    }
    elsif ($type eq 'DATE')
    {
        $length = 10;
    }
    elsif ($type =~ /BOOL/i)
    {
        $length = 1;
    }

    return ($length);
}

=head2 procedure RetrieveFieldAndValue

 Title		  : RetrieveFieldAndValue
 Usage		  : &RetrieveFieldAndValue($table,$rh_schema)
 Prerequisite     : &GetSchemaStructure
 Function	  : return an array that contains fields of 'ORF' and an hash (field => list of its values)
 Returns	  : a reference to an array of fields, and a reference to an hash  (field => list of its values)
 Args		  : $table
 Globals	  : none

=cut

sub RetrieveFieldAndValue
{
    my ($table, $rh_schema) = (@_);
    my %h_field_value = ();

    my $ra_fields = &RetrieveFields($table, $rh_schema);

    foreach my $field (@{$ra_fields})
    {
        if ($field =~ /.+_id$/)
        {
            my $rh_list_value = &RetrieveFieldValuesById($table, $field);

            $h_field_value{$field} = $rh_list_value;
        }
    }

    return ($ra_fields, \%h_field_value);

}

=head2 procedure RetrieveFieldValuesById

 Title		  : RetrieveFieldValuesById
 Usage		  : $rh_field_values = &RetrieveFieldValuesById($table, $field)
 Prerequisite : none
 Function	  : retrieve all the $field values contained in $table in the database by 'id'
 Returns	  : a reference to an hash (key:id, value:field_value)
 Args		  : $table: table of the data base
 		        $field: field we want to know all the data
 Globals	  : none

=cut

sub RetrieveFieldValuesById
{
    my ($table, $field) = (@_);
    my $module = &UseModule($table);

    my %h_field_values = ();
    my $ra_fetchall    = $module->fetchall();

    #ne selectionner que les valeurs de $champ
    # pour toutes les references vers les hash
    foreach my $rh_record (@{$ra_fetchall})
    {
        if ((defined $rh_record->{'id'}) && (defined $rh_record->{$field}))
        {
            $h_field_values{$rh_record->{'id'}} = $rh_record->{$field};
        }
    }

    return (\%h_field_values);
}

#
#	FONCTIONS DE JOINTURES
#

=head2 procedure N2Njoin

 Title		  : N2Njoin
 Usage		  : $rh_field_values = &N2Njoin($main_table,$joined_table)
 Prerequisite : none
 Function	  : retrieve all the $field values contained in $table in the database by 'id'
 Returns	  : a reference to an hash (key:id, value:field_value)
 Args		  : $table: table of the data base
 		        $field: field we want to know all the data
 Globals	  : none

=cut

sub N2Njoin
{
    my ($main_table, $joined_table, $joining_table, $ra_main_table_tuples, $cgi_target_script) = (@_);
    my @a_main_table_tuples_complete = ();
    my @a_fields                     = ();
    my $rh_schema                    = &GetSchemaStructure();
    my $main_field                   = &RetrieveMainField($joined_table, $rh_schema);
    
    my %h_main_fields_rh_liste_valeurs = ();
    if (ref($main_field) eq 'ARRAY')
    {
        push (@{$main_field}, 'id');
    }
    else
    {
        $main_field = ['id',$main_field];
    }
    
    if (ref($main_field) eq 'ARRAY')
    {
        foreach my $main_field (@{$main_field})
        {
            my $rh_liste_valeurs = &RetrieveFieldValuesById($joined_table, $main_field);
            $h_main_fields_rh_liste_valeurs{$main_field} = $rh_liste_valeurs;
        }
    }
    else
    {
        my $rh_liste_valeurs = &RetrieveFieldValuesById($joined_table, $main_field);
        $h_main_fields_rh_liste_valeurs{$main_field} = $rh_liste_valeurs;
    }
    
    my @a_joining_table_main_fields = ();
    $main_field = &RetrieveMainField($joining_table, $rh_schema);
    if (ref($main_field) eq 'ARRAY')
    {
        foreach my $main_field (@{$main_field})
        {
            push(@a_joining_table_main_fields, $main_field);
        }
    }
    else
    {
        push(@a_joining_table_main_fields, $main_field);
    }
    
    foreach my $rh_main_table_tuple (@{$ra_main_table_tuples})
    {
        my %h_main_table_tuple_complete = %{$rh_main_table_tuple};
        
        my $module = &UseModule($joining_table);
        
        my $ra_joining_table_matches =
        $module->fetchbySQL($main_table . "_id = '" . $rh_main_table_tuple->{'id'} . "'");
        
        foreach my $main_field_joined_table (keys %h_main_fields_rh_liste_valeurs)
        {
            my @a_empty = ();
            $h_main_table_tuple_complete{"$joined_table.$main_field_joined_table"} = \@a_empty;
        }
        
        if ($#a_fields < 0)
        {
            @a_fields = (keys(%h_main_table_tuple_complete));
        }
        if ($#{$ra_joining_table_matches} >= 0)
        {
            foreach my $rh_joining_table_match (@{$ra_joining_table_matches})
            {
                foreach my $main_field_joined_table (keys %h_main_fields_rh_liste_valeurs)
                {
			
                    my $displayed_value = $h_main_fields_rh_liste_valeurs{$main_field_joined_table}->{$rh_joining_table_match->{$joined_table . '_id'}};
		    
                    if ((defined $cgi_target_script) && ($cgi_target_script ne ''))
                    {
                        use HTML::Entities;
                        my $encoded_value = encode_entities($displayed_value);
                        $displayed_value = "<a href=\"$cgi_target_script$encoded_value\">$displayed_value</a>";
                    }
                    push(@{$h_main_table_tuple_complete{"$joined_table.$main_field_joined_table"}}, $displayed_value);
                }
                foreach my $joining_table_main_field (@a_joining_table_main_fields)
                {
                    push(
                        @{$h_main_table_tuple_complete{"$joining_table.$joining_table_main_field"}},
                        $rh_joining_table_match->{$joining_table_main_field}
                        );
                }
                
            }
            
        }
        push(@a_main_table_tuples_complete, \%h_main_table_tuple_complete);
    }
    
    return (\@a_fields, \@a_main_table_tuples_complete);
    
}

sub N21join
{
    my ($ra_main_table_tuples, $ra_fields, $rh_joined_tables_links) = (@_);
    my @a_main_table_tuples_complete = ();
    my @a_fields                     = ();
    my $rh_schema                    = &GetSchemaStructure();
    
    my %h_joined_tables = ();
    my $is_joined       = 0;
    foreach my $field (@{$ra_fields})
    {
        
        if ($field =~ /[\w+\.]_id$/)
        {
            my ($joined_table) = ($field =~ /(\w+)_id$/);
            
            my $main_field       = &RetrieveMainField($joined_table,       $rh_schema);
            my $rh_liste_valeurs = &RetrieveFieldValuesById($joined_table, $main_field);
            if (ref($main_field) eq 'ARRAY')
            {
                push (@{$main_field}, 'id');
            }
            else
            {
                $main_field = ['id',$main_field];
            }
            
            if (ref($main_field) eq 'ARRAY')
            {
                foreach my $main_field (@{$main_field})
                {
                    my $rh_liste_valeurs = &RetrieveFieldValuesById($joined_table, $main_field);
                    push (@{$h_joined_tables{$field}},{joined_table => $joined_table, main_field => $main_field, values_list_hashref => $rh_liste_valeurs});
                }
    	    }
            #else
    	    #{
            #    my $rh_liste_valeurs = &RetrieveFieldValuesById($joined_table, $main_field);
            #	$h_joined_tables{$field} = {joined_table => $joined_table, main_field => $main_field, values_list_hashref => $rh_liste_valeurs};
            #}
            
            $is_joined = 1;
        }
    }
    
    if ($is_joined)
    {
        foreach my $rh_main_table_tuple (@{$ra_main_table_tuples})
        {
            my %h_main_table_tuple_complete = %{$rh_main_table_tuple};
            foreach my $joining_field (keys %h_joined_tables)
            {
                
                if (ref ($h_joined_tables{$joining_field}) =~ /array/i)
                {
                    foreach my $item (@{$h_joined_tables{$joining_field}})
                    {
                        my $joined_table     = $item->{'joined_table'};
                		my $main_field       = $item->{'main_field'};
                		my $rh_liste_valeurs = $item->{'values_list_hashref'};
                        if ($h_main_table_tuple_complete{$joining_field} ne '')
                		{
                		    if (ref($h_main_table_tuple_complete{$joining_field}) eq 'ARRAY')
                		    {
                        		foreach my $item (@{$h_main_table_tuple_complete{$joining_field}})
                        		{
                        		    
                                    if (defined $rh_joined_tables_links->{$joined_table})
                                    {
                                        my $wb_target_function = $rh_joined_tables_links->{$joined_table}->{'wb_function'};
                                        my $cgi_target_script  = $rh_joined_tables_links->{$joined_table}->{'cgi_script'};
                                        $cgi_target_script =~ s/MASKWB_FUNCTION/$wb_target_function/g;
                                        $rh_liste_valeurs->{$item} = '<a href="'. $cgi_target_script. $rh_liste_valeurs->{$item} . '">' . $rh_liste_valeurs->{$item}.'</a>';
                                    }
                                    push(
                                        @{$h_main_table_tuple_complete{"$joined_table.$main_field"}},
                                        $rh_liste_valeurs->{$item}
                                        );
                                    
                        		}
                		    }
                		    else
                		    {
                                
                        		if (defined $rh_joined_tables_links->{$joined_table})
                                {
                                    my $wb_target_function = $rh_joined_tables_links->{$joined_table}->{'wb_function'};
                                    my $cgi_target_script  = $rh_joined_tables_links->{$joined_table}->{'cgi_script'};
                                    $cgi_target_script =~ s/MASKWB_FUNCTION/$wb_target_function/g;
                                    if ($rh_liste_valeurs->{$h_main_table_tuple_complete{$joining_field}} !~ /href/)
                                    {
                                        $rh_liste_valeurs->{$h_main_table_tuple_complete{$joining_field}} = '<a href="'. $cgi_target_script. $rh_liste_valeurs->{$h_main_table_tuple_complete{$joining_field}} . '">' . $rh_liste_valeurs->{$h_main_table_tuple_complete{$joining_field}}.'</a>';
                                    }
                                }
                                
                                $h_main_table_tuple_complete{"$joined_table.$main_field"} =
                                $rh_liste_valeurs->{$h_main_table_tuple_complete{$joining_field}};
                                
                		    }
                		}
                        
                    }
                    
                }
                else
                {
                	my $joined_table     = $h_joined_tables{$joining_field}->{'joined_table'};
                	my $main_field       = $h_joined_tables{$joining_field}->{'main_field'};
                	my $rh_liste_valeurs = $h_joined_tables{$joining_field}->{'values_list_hashref'};
                	if ($h_main_table_tuple_complete{$joining_field} ne '')
                	{
                	    if (ref($h_main_table_tuple_complete{$joining_field}) eq 'ARRAY')
                	    {
                        	foreach my $item (@{$h_main_table_tuple_complete{$joining_field}})
                        	{
                        	    push(
                                    @{$h_main_table_tuple_complete{"$joined_table.$main_field"}},
                                    $rh_liste_valeurs->{$item}
                                    );
                                
                        	}
                	    }
                	    else
                	    {
                        	$h_main_table_tuple_complete{"$joined_table.$main_field"} =
                            $rh_liste_valeurs->{$h_main_table_tuple_complete{$joining_field}};
                            
                	    }
                	}
                }
                
            }
            if ($#a_fields < 0)
            {
                @a_fields = (keys(%h_main_table_tuple_complete));
            }
            push(@a_main_table_tuples_complete, \%h_main_table_tuple_complete);
        }
    }
    
    return (\@a_fields, \@a_main_table_tuples_complete);
    
}


sub One2Njoin
{
    my ($main_table, $joined_table, $ra_main_table_tuples) = (@_);
    my @a_main_table_tuples_complete = ();
    my @a_fields                     = ();
    my $rh_schema                    = &GetSchemaStructure();
    my $main_field                   = &RetrieveMainField($joined_table, $rh_schema);

    my %h_main_fields_rh_liste_valeurs = ();
    if (ref($main_field) eq 'ARRAY')
    {
	push (@{$main_field}, 'id');
    }
    else
    {
	$main_field = ['id',$main_field];
    }

    if (ref($main_field) eq 'ARRAY')
    {
        foreach my $main_field (@{$main_field})
        {
	    my $rh_liste_valeurs = &RetrieveFieldValuesById($joined_table, $main_field);
            $h_main_fields_rh_liste_valeurs{$main_field} = $rh_liste_valeurs;
        }
    }
    else
    {
        my $rh_liste_valeurs = &RetrieveFieldValuesById($joined_table, $main_field);
        $h_main_fields_rh_liste_valeurs{$main_field} = $rh_liste_valeurs;
    }

    foreach my $rh_main_table_tuple (@{$ra_main_table_tuples})
    {
        my %h_main_table_tuple_complete = %{$rh_main_table_tuple};

        my $module                  = &UseModule($joined_table);
        my $ra_joined_table_matches = $module->fetchbySQL($main_table . "_id = '" . $rh_main_table_tuple->{'id'} . "'");
	foreach my $main_field_joined_table (keys %h_main_fields_rh_liste_valeurs)
        {
            my @a_empty = ();
            $h_main_table_tuple_complete{"$joined_table.$main_field_joined_table"} = \@a_empty;
        }

        if ($#a_fields < 0)
        {
            @a_fields = (keys(%h_main_table_tuple_complete));
	    
        }
        if ($#{$ra_joined_table_matches} >= 0)
        {
	
            foreach my $rh_joined_table_match (@{$ra_joined_table_matches})
            {

                foreach my $main_field_joined_table (keys %h_main_fields_rh_liste_valeurs)
                {
                    my $displayed_value =
                      $h_main_fields_rh_liste_valeurs{$main_field_joined_table}->{$rh_joined_table_match->{'id'}};
		                  
                    push(@{$h_main_table_tuple_complete{"$joined_table.$main_field_joined_table"}}, $displayed_value);
                }
            }
        }

        push(@a_main_table_tuples_complete, \%h_main_table_tuple_complete);
    }

    return (\@a_fields, \@a_main_table_tuples_complete);

}

#
#	FONCTIONS DE GESTION IDENTIFICATION
#

=head2 function IsGuest

 Title		  : IsGuest
 Usage		  : $boolean = &IsGuest($rh_param, $o_cgi)
 Prerequisite : none
 Function	  : Test if the crypted privilege cookie correspond to the 'guest' privilege
 Returns	  : 0/1
 Args		  : $rh_param: ParamParser structure
 		    $o_cgi: CGI object
 Globals	  : $COOKIE_PRIVILEGE_PARAM_NAME

=cut

sub IsGuest
{
    my ($rh_param, $o_cgi) = (@_);
    my $crypted_privilege = $o_cgi->cookie($rh_param->Get($COOKIE_PRIVILEGE_PARAM_NAME));
    if (($crypted_privilege eq &CryptCookieValue('guest')) || ($crypted_privilege eq ''))
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

=head2 function GetUserId

 Title		  : GetUserId
 Usage		  : $user_id = &GetUserId($rh_param, $o_cgi)
 Prerequisite     : none
 Function	  : retrieve user_id from cookie
 Returns	  : 0/1
 Args		  : $rh_param: ParamParser structure
 		    $o_cgi: CGI object
 Globals	  : $COOKIE_USERID_PARAM_NAME

=cut

sub GetUserId
{
    my ($rh_param, $o_cgi) = (@_);
    my $user_id = $o_cgi->cookie($rh_param->Get($COOKIE_USERID_PARAM_NAME));
    if ((defined $user_id) && ($user_id ne ''))
    {
        return $user_id;
    }
    else
    {
        return 0;
    }
}

=head2 function CryptCookieValue

 Title		  : CryptCookieValue
 Usage		  : $crypted_value = &CryptCookieValue($value)
 Prerequisite     : none
 Function	  : applies MD5 encryption onto a value
 Returns	  : MD5 crypted value
 Args		  : $rh_param: ParamParser structure
 		    $o_cgi: CGI object
 Globals	  : none

=cut

sub CryptCookieValue
{
    my $value         = shift();
    my $crypted_value = Digest::MD5::md5_hex($value);
    return $crypted_value;
}

#
# SEARCH FUNCTIONS
#

=head2 function GetFormContent

 Title    : GetFormContent
 Usage    : my $form_content = GetFormContent($template);
 Function : Utilise le template pour construire et renvoyer le contenu du formulaire
 Args     : $template  Le template a remplir
            $edit (optionnel) Si 1, on passe en mode edition (cf. DisplayInputField)
            $o_display_field (optionnel) Un objet de type $o_display_field (used by the template)
 Return   : Le code html
 Globals  : $O_PARAM, $O_CFG, $O_LOGGER

=cut
sub GetFormContent
{
    my ($template,$edit,$o_display_output_field) = (@_);
    $edit=0 unless(defined $edit);
    
    my $o_cfg = $main::O_CFG;
    
    my $o_display_input_field = New DisplayInputField($main::O_CFG,$main::O_LOGGER,$main::O_PARAM,\%H_SCHEMA_DB);
    $o_display_input_field->SetLogger($main::O_LOGGER);
    $o_display_input_field->SetEditMode() if ($edit==1);
    my $o_tmpl = GetTemplate($template);
    $o_tmpl->AddObject('DISPLAYINPUTFIELD',$o_display_input_field);
    $o_tmpl->AddObject('CFG',$o_cfg);
    $o_tmpl->AddObject('PARAM',$main::O_PARAM);
    my $img_http_dir = $o_cfg->Get('SERVERNAME') . '/' . # Copyright INRA/CNRS
# 
# Emmanuel.Courcelle@toulouse.inra.fr
# Sebastien.Carrere@toulouse.inra.fr
# 
# Ce logiciel est un programme informatique servant à [rappeler les
# caractéristiques techniques de votre logiciel]. 
# 
# Ce logiciel est régi par la licence [CeCILL|CeCILL-B|CeCILL-C] soumise au droit français et
# respectant les principes de diffusion des logiciels libres. Vous pouvez
# utiliser, modifier et/ou redistribuer ce programme sous les conditions
# de la licence [CeCILL|CeCILL-B|CeCILL-C] telle que diffusée par le CEA, le CNRS et l'INRIA 
# sur le site "http://www.cecill.info".
# 
# En contrepartie de l'accessibilité au code source et des droits de copie,
# de modification et de redistribution accordés par cette licence, il n'est
# offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
# seule une responsabilité restreinte pèse sur l'auteur du programme,  le
# titulaire des droits patrimoniaux et les concédants successifs.
# 
# A cet égard  l'attention de l'utilisateur est attirée sur les risques
# associés au chargement,  à l'utilisation,  à la modification et/ou au
# développement et à la reproduction du logiciel par l'utilisateur étant 
# donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
# manipuler et qui le réserve donc à des développeurs et des professionnels
# avertis possédant  des  connaissances  informatiques approfondies.  Les
# utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
# logiciel à leurs besoins dans des conditions permettant d'assurer la
# sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
# à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
# 
# Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
# pris connaissance de la licence [CeCILL|CeCILL-B|CeCILL-C], et que vous en avez accepté les
# termes.

$o_cfg->Get('HTTP_ROOT') . $o_cfg->Get('HTTP_IMG_DIR');
    $o_tmpl->AddVariable('IMG_HTTP_DIR',$img_http_dir);
    
    if (defined $o_display_output_field)
    {
        $o_tmpl->AddObject('DISPLAYOUTPUTFIELD',$o_display_output_field);
    }
    return $o_tmpl->Parse();
}    


=head2 function BuildWhereCondition

 Title    : BuildWhereCondition (was BuildSQLQuery)
 Usage    : BuildWhereCondition($o_param,$
 Function : Build the where condition, using the parameters field:xxx from $o_param
            We build a complete SQL statement, to retrieve the list of main table's IDs corresponding the parameters
            Then we return the corresponding where condition (a set of IDs)
 Args     : $o_param
            $main_table The name of the main table we are working on
            $rh_joined_tables A hash of hash: key = field_name->'joining_table', val = the jointure table
            $rh_joining_table A hash: key = field_name val = 1
 Return   : the where condition
 Globals  : $O_LOGGER

=cut
sub BuildWhereCondition
{
    my ($o_param, $main_table, $rh_joined_tables, $rh_joining_table) = (@_);
    my $o_logger = $main::O_LOGGER;
    
    my @a_table_list      = ($main_table);
    my @a_where_statement = ();
    
    # Foreach key from $o_param in relation with a field
    foreach my $key ($o_param->GetKeys('field:.+'))
    {
        # Skip that key if the value is empty
        my $val = $o_param->Get($key);
        next if ($val =~ /^ *$/);

        # Parse the key name, discovering the table and the field
        my ($target_table,$field);
        if ($key =~ /^field:(\w+)\.(\w+)$/)
        {
            $target_table = $1;
            $field        = $2;
        }
        elsif ($key =~ /^field:(\w+)$/)
        {
            $target_table = $main_table;
            $field        = $1;
        }
        else
        {
            next;   # should never happen
        }

        # Compute the test_operator, depends on the field type
        my $field_type = &GetTableFieldType(\%H_SCHEMA_DB,$target_table,$field);
        my $test_operator;
        if (($field_type eq 'INTEGER') || ($field_type eq 'REAL') || ($field_type eq 'DATE')) 
        {
            $test_operator = '=';
        }
        else
        {
            $test_operator = 'ILIKE';
        }
        
        if ($field_type eq 'DATE')
        {
            if (($val =~ /aaaa/) || ($val =~ /yyyy/))
            {
                next;
            }
        }

        # If the field must be looked for through a jointure table
        if (exists $rh_joined_tables->{$target_table})
        {
            if (!grep (/^$rh_joined_tables->{$target_table}->{'joining_table'}$/, @a_table_list))
            {
                push(@a_table_list, $rh_joined_tables->{$target_table}->{'joining_table'});
            }
            push(@a_table_list, $target_table);
            push(@a_where_statement, "\"$target_table\".\"$field\" = \'$val\'");
            push(@a_where_statement,
                "\""
                . $rh_joined_tables->{$target_table}->{'joining_table'} . "\".\""
                . $target_table
                . "_id\" = \"$target_table\".\"id\"");
            push(@a_where_statement,
                "\""
                . $rh_joined_tables->{$target_table}->{'joining_table'} . "\".\""
                . $main_table
                . "_id\" = \"$main_table\".\"id\"");
        }
        
        else
        {
            # If the field comes from another table, but it is not a jointure
            if (($target_table ne $main_table) && (!exists $rh_joining_table->{$target_table}))
            {
                push(@a_table_list,      $target_table);
                push(@a_where_statement, "$main_table.$target_table" . "_id = $target_table.id");
                push(@a_where_statement, "$target_table.$field = '$val'");
            }
            # ???
            elsif (exists $rh_joining_table->{$target_table})
            {
                if (!grep (/^$target_table$/, @a_table_list))
                {
                    push(@a_table_list, $target_table);
                }
                
                push(@a_where_statement,
                    "\"$target_table\".\"$field\" $test_operator \'"
                    . $val
                    . "\'");
                push(@a_where_statement,
                    "\"" . $target_table . "\".\"" . $main_table . "_id\" = \"$main_table\".\"id\"");
            }
            
            # ???
            else
            {
                push(@a_where_statement,
                    "\"$target_table\".\"$field\" $test_operator \'"
                    . $val
                    . "\'");
            }
        }

    }
    
    # Execute the sql query
    my $where_condition = join(' AND ', @a_where_statement);
    if (defined $where_condition and $where_condition ne '')
    {
        my $sql_query =
        "SELECT \"$main_table\".\"id\" FROM \""
        . join('","', @a_table_list)
        . '" WHERE '
        . $where_condition;
        
        $o_logger->Trace("BuildWhereCondition now Executing the SQL request $sql_query\n",'*','3');
        my $sth   = $clism_DBH->prepare($sql_query);
        $sth->execute;
        my @a_ids = ();
        while (my $id = $sth->fetchrow_array)
        {
            push(@a_ids, $id);
        }
        $sth->finish;
        $o_logger->Trace("SQL returned - " . @a_ids . " row returned\n",'*','3');
        if (@a_ids==0)
        {
            return '';
        }
        else
        {
            my $rvl = "\"$main_table\".\"id\" IN ('" . join("','", @a_ids) . "')";
            $o_logger->Trace("BuildWhereCondition returning $rvl\n",'*','3');
            return $rvl;
        };
    }
    else
    {
        $o_logger->Trace("BuildWhereCondition returning nothing (where_condition undefined)\n",'*','3');
        return '';
    }
}



sub HiddenSessionParameters
{
	my $rh_param = shift;
	my $hidden = '';

	my @a_list_parameters = ('__wb_schema','__wb_login','__wb_lock');
	foreach my $parameter (@a_list_parameters)
	{
		if ($rh_param->IsDefined($parameter))
		{
			$hidden .= '<input type="hidden" name="'.$parameter.'" value="' .$rh_param->Get($parameter) .'"/>';
		}
	}
	return $hidden;

}

sub UrlSessionParameters
{
	my $rh_param = shift;
	my $url = '';

	my @a_list_parameters = ('__wb_schema','__wb_login','__wb_lock');
	foreach my $parameter (@a_list_parameters)
	{
		if ($rh_param->IsDefined($parameter))
		{
			$url .= '&amp;'.$parameter.'=' .$rh_param->Get($parameter);
		}
	}
	return $url;

}

1;


