# Copyright INRA/CNRS
# 
# Emmanuel.Courcelle@toulouse.inra.fr
# Sebastien.Carrere@toulouse.inra.fr
# 
# Ce logiciel est un programme informatique servant à [rappeler les
# caractéristiques techniques de votre logiciel]. 
# 
# Ce logiciel est régi par la licence [CeCILL|CeCILL-B|CeCILL-C] soumise au droit français et
# respectant les principes de diffusion des logiciels libres. Vous pouvez
# utiliser, modifier et/ou redistribuer ce programme sous les conditions
# de la licence [CeCILL|CeCILL-B|CeCILL-C] telle que diffusée par le CEA, le CNRS et l'INRIA 
# sur le site "http://www.cecill.info".
# 
# En contrepartie de l'accessibilité au code source et des droits de copie,
# de modification et de redistribution accordés par cette licence, il n'est
# offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
# seule une responsabilité restreinte pèse sur l'auteur du programme,  le
# titulaire des droits patrimoniaux et les concédants successifs.
# 
# A cet égard  l'attention de l'utilisateur est attirée sur les risques
# associés au chargement,  à l'utilisation,  à la modification et/ou au
# développement et à la reproduction du logiciel par l'utilisateur étant 
# donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
# manipuler et qui le réserve donc à des développeurs et des professionnels
# avertis possédant  des  connaissances  informatiques approfondies.  Les
# utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
# logiciel à leurs besoins dans des conditions permettant d'assurer la
# sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
# à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 
# 
# Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
# pris connaissance de la licence [CeCILL|CeCILL-B|CeCILL-C], et que vous en avez accepté les
# termes.

package UpdateTable;

use strict;
use ParamParser;
use WebBuilder;
use File::Basename;
use AnotherTemplate;
use O2DBI::O2DBI_WebUtils;

use O2DBI::DisplayTable;

use LipmObject;
use base qw(LipmObject);

use Error qw( :try );
use LipmError;

=head2 Function _Init (the constructor)
 Title            : _Init
 Usage            : my $o_obj = New UpdateTable($o_cfg, $o_logger)
 Function         : threw the New function of LipmObject, builds the object
 Returns          : The object
 Args             : $o_cfg   = A ParamParser containing the configuration parameters
                    $o_logger= A Logger object (optional)
                    $o_param
 Globals          : none

=cut
sub _Init
{
    my $self = shift;
    my ($o_cfg, $o_logger, $o_param) = (@_);
    
    $self -> {_o_cfg}   = $o_cfg;
    $self -> SetLogger($o_logger) if (defined $o_logger);
    $self -> {_o_param} = $o_param;
    
    return $self;
}

=head2 Procedure SetTableName, Function GetTableName
 Title  : SetTableName, GetTableName
 Usage  : Setter, Getter

=cut

sub SetTableName
{
    my $self = shift;
    
    my ($table_name) = (@_);
    $self->{__table} = $table_name;
}
sub GetTableName
{
    my $self = shift;
    
    return $self->{__table};
}


=head2 Procedure UpdateTable
 Title       : UpdateTable
 Usage       : $o_obj->UpdateTable($o_web_builder)
 Procedure   : Insert a row, Delete a row, or update an existing row of the table... or of a jointure table
               Then, print the form for the next step
 Args        : $o_web_builder
 Globals     : none

=cut
sub UpdateTable
{
    my $self = shift;
    
    my ($o_wb)     = (@_);
    $self->__UpdateTable($o_wb,0);
}
sub UpdateSecondaryTable
{
    my $self = shift;
    
    my ($o_wb)     = (@_);
    $self->__UpdateTable($o_wb,1);
}    
    
sub __UpdateTable
{
    my $self = shift;
    
    my ($o_wb,$secondary_flg)     = (@_);
    my $o_param    = $o_wb->GetObjectParam();
    my $main_table = $self->{__table};
    my $o_cfg      = $self->{_o_cfg};
    
    #my $tmp_out;
    #$o_param->Dump('STRING',\$tmp_out,"");
    #warn $tmp_out;

    # If cstep is defined, we come from a previous screen: try to insert or update data
    my $step = $o_param->IsDefined('step') ? $o_param->Get('step') : 10;
    if ($o_param->IsDefined('cstep'))
    {
        # Insert data in some table's database - the table is main_table, unless redefined by $o_param
        try
        {
            my $table = ($o_param->IsDefined('table')) ? $o_param->Get('table') : $main_table;
            my $id = &O2DBI::InsertForms::InsertNewElement($o_param, $table);            
            $self->_Trace("Table $table, element $id created/updated\n",'*',2);
        }
        
        catch LipmError with
        {
            my ($err,$r) = @_;
            
            # The legal values for cstep are: 2O, 2R where 2 is the current step number, O means 'Optional', R 'Required'
            my ($cstep,$opt_flg) = ($o_param->Get('cstep') =~ /(\d+)([OR])/);
            
            #warn "koukou #$cstep#$opt_flg#".$o_param->Get('cstep')."#"; 
            
            # This update is required: display the error and return to the current step
            if ($opt_flg eq 'R')
            {
                warn $err->Explain(1);
                print '<div class="wb_error">'.$err->text().'</div>';
                $o_param->Set('step',$cstep);
                $step = $cstep;
            }
            
            # This update is optional, but if the user pressed "same screen" we display the error message
            else
            {
                if ($step==$cstep)
                {
                    print '<div class="wb_error">'.$err->text().'</div>';
                }
            }
        };
    }

    # Print the form
    my $html = "";
    my $template = $main_table . "_inserttemplate$step";
    my $o_display_table = $self->{_o_display_table};

    # We are updating a "secondary" table: at top of screen the new values, at bottom the complete table
    # The method used to display the complete table are in the Displayfield/xxxField objects, they are called from the template
    if ($secondary_flg==1)
    {
        my $o_display_output_field = $o_display_table->GetDisplayField();
        $html                      = &O2DBI::O2DBI_WebUtils::GetFormContent($template,1,$o_display_output_field);

        # Do not include the buttons for the FINISHED screen (step 1000)
        $html   .= &O2DBI::O2DBI_WebUtils::GetFormContent($main_table . '_menu',1) if ($step != 1000);

        # Print the form
        print $o_wb->GetForm("",$html);
    }
    
    # We are updating a "main" table, ie souche, plasmide or biblio: at top of screen the new values with some values to delete,
    # at bottom the current row is displayed if its id is known
    else
    {
        my $o_display_output_field = undef;
        my $ra_o_results    = "";
        my $id = $o_param->Get('field:'.$main_table.'.id');

        # If id is known, we could have some output to display, even from the form (for deletion)
        if ($id ne '')
        {
            $ra_o_results = $o_display_table -> SearchData($main_table.".id = '$id'");
            $o_display_output_field = $o_display_table->GetDisplayField();
            $o_display_output_field->SetRow($ra_o_results->[0]);
        };
        my $html = &O2DBI::O2DBI_WebUtils::GetFormContent($template,1,$o_display_output_field);
        
        # Do not include the buttons for the FINISHED screen (step 1000)
        $html   .= &O2DBI::O2DBI_WebUtils::GetFormContent($main_table . '_menu',1) if ($step != 1000);

        # Print the form
        print $o_wb->GetForm("",$html);

        # If id is known, print the current data
        if ($id ne '')
        {
            my $output_template = $main_table.'_template';
            print $o_display_table->DisplayRow($output_template, $ra_o_results->[0]);
        }
    }
}

1;

