package UpdateSouche;

use strict;
use WebBuilder;
#use File::Basename;
#use AnotherTemplate;
#use O2DBI::O2DBI_WebUtils;
#use O2DBI::DisplayField;

use O2DBI::DisplayTable::DisplaySouche;

use O2DBI::UpdateTable;
use base qw(UpdateTable);

=head2 Function _Init (the constructor)
 Title            : _Init
 Usage            : my $o_obj = New UpdateTable($o_cfg, $o_logger)
 Function         : build the object, initializing a DisplaySouche object
 Returns          : The object
 Args             : $o_cfg   = A ParamParser containing the configuration parameters
                    $o_logger= A Logger object (optional)
 Globals          : none

=cut
sub _Init
{
    my $self = shift;
    my ($o_cfg, $o_logger, $o_param) = (@_);
    
    $self->SUPER::_Init(@_);
    $self -> SetTableName('souche');
    my $o_display_souche = New DisplaySouche(@_);
    $self -> { _o_display_table } = $o_display_souche;
    
    return $self;
}


=head2 Procedure UpdateTable
 Title       : UpdateTable
 Usage       : $o_obj->UpdateTable($o_web_builder)
 Procedure   : If necessary, Set the parameter 'code', then call the UpdateTable version of this function
 Args        : $o_web_builder
 Globals     : none

=cut
sub UpdateTable
{
    my $self = shift;

    my ($o_wb)     = (@_);
    my $o_param    = $o_wb->GetObjectParam();
 
    # Used only to display
    if (!$o_param->IsDefined('code'))
    {
        $o_param->Set('code',$o_param->Get('field:souche.code'));
    }
    $self->SUPER::UpdateTable(@_);
}
    
    
 
=head2 Procedure UpdateExistingRow
 Title       : UpdateExistingRow
 Usage       : $o_obj->UpdateExistingRow($o_web_builder)
 Procedure   : Update an existing row: try to know the id of the row to update from the code parameter which must be present in $o_param,
               then call the general purpose UpdateTable
 Args        : $o_web_builder
 Globals     : none

=cut
sub UpdateExistingRow
{
    my $self = shift;
    
    my ($o_wb)  = (@_);
    my $o_param = $o_wb->GetObjectParam();
    my $table   = 'souche';

    # If cstep is defined, we come from an existing from
    if ($o_param->IsDefined('cstep'))
    {
        # Search the id from the field code
        if ($o_param->IsDefined("field:${table}.code") and !$o_param->IsDefined("field:${table}.id"))
        {
            my $code = $o_param->Get("field:${table}.code");
            my $ra_data = &O2DBI::O2DBI_WebUtils::ExecuteFetchbySQL("code='$code'",$table);
            my $id = $ra_data->[0]->{id};
            $o_param->Set("field:${table}.id",$id);
            
            # Used only to display
            $o_param->Set("code",$code);

        };
        
        # Modify $o_param to enter the found id
        if ($o_param->IsDefined("field:${table}.id"))
        {
            $self->UpdateTable($o_wb);
            return;
        }
        
        # If id still unknown, we are in error
        else
        {
            print '<div class="wb_error">ERROR - Code unspecified</div>';
        }
    }

    # Print the form, starting from template nb 5
    my $template = "souche_inserttemplate5";
    my $form_html_content = &O2DBI::O2DBI_WebUtils::GetFormContent($template,1);
    print $o_wb->GetForm("",$form_html_content);
}

1;

