package SchemaDB;

# $Id$

### Lignes supprimees en Avril 2007 = nettoyage des donnees obsoletes

use strict;

BEGIN {
    use Exporter   ();
    our ($VERSION, @ISA, @EXPORT);
    
    $VERSION = do {my @r = (q$Rev$ =~ /\d+/g); $r[0]};
    @ISA     = qw(Exporter);
    
    # your exported package globals go here,
    # as well as any optionally exported functions
    @EXPORT      = qw($DB_NAME $COOKIE_PRIVILEGE_PARAM_NAME $COOKIE_USERID_PARAM_NAME %H_SCHEMA_DB);
}
our @EXPORT;

# exported package globals go here
our ($DB_NAME, $COOKIE_PRIVILEGE_PARAM_NAME, $COOKIE_USERID_PARAM_NAME, %H_SCHEMA_DB);

# non-exported package globals go here
#our @more;
#our $stuff;

##############################################################################
##################### SPECIFIQUE DE LA BASE DE DONNEES #######################
##############################################################################

$DB_NAME			         = 'clism';
$COOKIE_PRIVILEGE_PARAM_NAME = 'COOKIE_PRIVILEGE';
$COOKIE_USERID_PARAM_NAME	 = 'COOKIE_USERID';

#
# Le schema de la base
#
#    cf. le diagramme uml pour y comprendre quelque chose
#    Les attributs Toto_id sont des cles etrangeres, ce sont les associations du diagramme uml
#    Lorsqu'il y a des roles a preciser, ceux-ci sont indiques en commentaire
#

%H_SCHEMA_DB = (
                # Les trois tables principales
                
                'souche' => {
                    members => {
                        'code'        => 'VARCHAR(20)',
###                        'noms'        => 'TEXT',
                        'proprietes'  => 'TEXT',
                        'parents'     => 'VARCHAR(80)',
                        'construitpar'=> 'VARCHAR(80)',
                        'date'        => 'DATE',
                        'rdate'       => 'DATE',
                        'numeroboite' => 'VARCHAR(10)',
                        'colonneboite'=> 'CHAR',
                        'ligneboite'  => 'INTEGER',
                        'construction'=> 'TEXT',
###                        'obtention_id'=> 'INTEGER',
                        'espece_id'   => 'INTEGER',
                        'numero'      => 'INTEGER',
                        'gmi80'       => 'VARCHAR(20)',
###                        'genes'       => 'TEXT',
###                        'genotype'    => 'TEXT',
###                        'transposons' => 'TEXT'
                    },
                    creator     => ['code','espece_id'],
                    constructors => [['code'],['gmi80']],
###                    main => ['code','noms']
                    main => ['code']
                },
                
                'plasmide' => {
                    members => {
                        'code'                  => 'VARCHAR(20)',
###                        'noms'                  => 'TEXT',
                        'derivede'              => 'VARCHAR(80)',
                        'taille'                => 'REAL',
                        'construitpar'          => 'VARCHAR(80)',
                        'construction'          => 'TEXT',
                        'carterestriction'      => 'TEXT',
                        'groupeincompatibilite' => 'TEXT',
                        'numero'                => 'INTEGER',
                        'vecteur_id'            => 'INTEGER',
###                        'genes'                 => 'VARCHAR(80)',
###                        'genotype'              => 'TEXT',
                        'vnti_gb'               => 'BYTEA',
                        'vnti_jpg'              => 'BYTEA'
                    },
                    creator => ['code'],
                    constructors => [['code']],
###                    main => ['code','noms']
                    main => ['code']
                },
                
                'biblio' => {
                    members => {
                        'auteurs'      => 'TEXT',
                        'titre'        => 'TEXT',
                        'url'          => 'VARCHAR(255)',
                        'livre'        => 'TEXT',
                        'annee'        => 'INTEGER',
                        'volume'       => 'VARCHAR(80)',
                        'pages'        => 'VARCHAR(80)',
                        'motscles'     => 'TEXT',
                        'revue_id'     => 'INTEGER',
                        'editeur_id'   => 'INTEGER',
                        'numero'       => 'INTEGER'
                    },
                    creator => [],
                    main => 'titre'
                },
                
                'nom_souche' => {
                    members => {
                        'nom' => 'TEXT',
                        'souche_id' => 'INTEGER'
                    },
                    creator => ['nom','souche_id'],
                    constructors => [['nom']],
                    main	=> 'nom'
                },
                
                'nom_plasmide' => {
                    members => {
                        'nom' => 'TEXT',
                        'plasmide_id' => 'INTEGER'
                    },
                    creator => ['nom','plasmide_id'],
                    constructors => [['nom']],
                    main	=> 'nom'
                },
                
                # Les tables secondaires, liees aux tables principales par des relations 1-n
                
                'espece' => {
                    members => {
                        'nom'         => 'VARCHAR(80)'
                    },
                    creator     => ['nom'],
                    constructors => [['nom']],
                    main => 'nom'
                },
                
                'vecteur' => {
                    members => {
                        'nom'         => 'VARCHAR(40)'
                    },
                    creator     => ['nom'],
                    constructors => [['nom']],
                    main => 'nom'
                },
                    
                'revue' => {
                    members => {
                        'nom'         => 'VARCHAR(80)'
                    },
                    creator     => ['nom'],
                    constructors => [['nom']],
                    main => 'nom'
                },
                
                'editeur' => {
                    members => {
                        'nom'         => 'VARCHAR(80)'
                    },
                    creator     => ['nom'],
                    constructors => [['nom']],
                    main => 'nom'
                },
                
                'siterestriction' => {
                    members => {
                        'nom'         => 'VARCHAR(80)',
                        'description' => 'TEXT'
                    },
                    creator     => ['nom'],
                    constructors => [['nom']],
                    main	=> 'nom'
                },
                
                'fragment' => {
                    members => {
                        'id'        => 'INTEGER',
                        'source'             => 'VARCHAR(80)',
                        'taille'             => 'REAL',
                        'plasmide_id'        => 'INTEGER',
                        'siterestriction_id' => 'INTEGER'       # Role = extremite
                    },
                    creator     => ['taille','plasmide_id'],
                    main	=> ['id','taille','siterestriction_id','source'],
		    constructors => [['id']],
                },
                
                'phenotypeconfere' => {
                    members => {
                        'concentration' => 'VARCHAR(15)',
                        'commentaire' => 'VARCHAR(80)',
                        'caracterep_id'  => 'INTEGER',
                        'plasmide_id'   => 'INTEGER'
                    },
                    creator     => ['caracterep_id','plasmide_id'],
                    constructors=> [['caracterep_id','plasmide_id']],
                    main	=> 'concentration'
                },
                
                'obtention' => {
                    members => {
                        'nom'         => 'VARCHAR(80)',
                    },
                    creator     => ['nom'],
                    constructors => [['nom']],
                    main	=> 'nom'
                },
                
                'antibios' => {
                    members => {
                        'concentration' => 'VARCHAR(15)',
                        'commentaire' => 'VARCHAR(80)',
                        'caracterep_id'  => 'INTEGER',
                        'souche_id'   => 'INTEGER'
                    },
                    creator     => ['caracterep_id','souche_id'],
                    constructors=> [['caracterep_id','souche_id']],
                    main	=> ['caracterep_id','concentration']
                },
                
                'caracterep' => {
                    members => {
                        'nom'         => 'VARCHAR(80)',
                        'description' => 'TEXT',
                        'antibio'     => 'BOOLEAN'
                    },
                    creator     => ['nom'],
                    constructors => [['nom']],
                    main	=> 'nom'
                },
                
                'caracterea' => {
                    members => {
                        'nom'         => 'VARCHAR(80)',
                        'description' => 'TEXT'
                    },
                    creator     => ['nom'],
                    constructors => [['nom']],
                    main	=> ['nom','description']
                },
                
                'caractere' => {
                    members => {
                        'nom'         => 'VARCHAR(80)',
                        'description' => 'TEXT'
                    },
                    creator     => ['nom'],
                    constructors => [['nom']],
                    main	=> 'nom'
                },
                
                 'transposon' => {
                    members => {
                        'nom'         => 'VARCHAR(80)',
                        'description' => 'TEXT'
                    },
                    creator     => ['nom'],
                    constructors => [['nom']],
                    main	=> 'nom'
                },
                
                 'gene' => {
                    members => {
                        'nom'         => 'VARCHAR(80)',
                        'description' => 'TEXT'
                    },
                    creator     => ['nom'],
                    constructors => [['nom']],
                    main	=> 'nom'
                },
                
                # Les tables permettant d'implementer les relations n-n
                'biblio_souche' => {
                    members => {
                        'biblio_id'   => 'INTEGER',
                        'souche_id'   => 'INTEGER'
                    },
                    creator      => ['biblio_id','souche_id'],
                    constructors => [['biblio_id','souche_id']]
                },
                
                'biblio_plasmide' => {                      # CONTIENT UN COUPLE DE CLES ETRANGERES
                    members => {
                        'biblio_id' => 'INTEGER',
                        'plasmide_id'   => 'INTEGER'
                    },
                    creator      => ['biblio_id','plasmide_id'],
                    constructors => [['biblio_id','plasmide_id']]
                },
                
                'caractere_plasmide' => {                      # CONTIENT UN COUPLE DE CLES ETRANGERES
                    members => {
                        'caractere_id' => 'INTEGER',
                        'plasmide_id'   => 'INTEGER'
                    },
                    creator      => ['caractere_id','plasmide_id'],
                    constructors => [['caractere_id','plasmide_id']]
                },
                
                'caractere_souche' => {                      # CONTIENT UN COUPLE DE CLES ETRANGERES
                    members => {
                        'caractere_id' => 'INTEGER',
                        'souche_id'   => 'INTEGER'
                    },
                    creator      => ['caractere_id','souche_id'],
                    constructors => [['caractere_id','souche_id']]
                },
                
                'caracterea_souche' => {                      # CONTIENT UN COUPLE DE CLES ETRANGERES
                    members => {
                        'caracterea_id' => 'INTEGER',
                        'souche_id'   => 'INTEGER'
                    },
                    creator      => ['caracterea_id','souche_id'],
                    constructors => [['caracterea_id','souche_id']]
                },
                
                'plasmide_souche' => {                        # CONTIENT UN COUPLE DE CLES ETRANGERES
                    members => {
                        'plasmide_id'   => 'INTEGER',
                        'souche_id' => 'INTEGER'
                    },
                    creator      => ['plasmide_id','souche_id'],
                    constructors => [['plasmide_id','souche_id']]
                },
                
                'transposon_souche' => {                      # CONTIENT UN COUPLE DE CLES ETRANGERES
                    members => {
                        'transposon_id' => 'INTEGER',
                        'souche_id'   => 'INTEGER'
                    },
                    creator      => ['transposon_id','souche_id'],
                    constructors => [['transposon_id','souche_id']]
                },

                'gene_souche' => {                      # CONTIENT UN COUPLE DE CLES ETRANGERES
                    members => {
                        'gene_id'   => 'INTEGER',
                        'souche_id' => 'INTEGER'
                    },
                    creator      => ['gene_id','souche_id'],
                    constructors => [['gene_id','souche_id']]
                },

                'transposon_plasmide' => {                      # CONTIENT UN COUPLE DE CLES ETRANGERES
                    members => {
                        'transposon_id' => 'INTEGER',
                        'plasmide_id'   => 'INTEGER'
                    },
                    creator      => ['transposon_id','plasmide_id'],
                    constructors => [['transposon_id','plasmide_id']]
                },

                'gene_plasmide' => {                      # CONTIENT UN COUPLE DE CLES ETRANGERES
                    members => {
                        'gene_id'   => 'INTEGER',
                        'plasmide_id' => 'INTEGER'
                    },
                    creator      => ['gene_id','plasmide_id'],
                    constructors => [['gene_id','plasmide_id']]
                },

                'utilisateur' => {
                    members => {
                        'description'=> 'VARCHAR(80)',
                        'login'      => 'VARCHAR(20)',
                        'mail'       => 'VARCHAR(80)',
                        'prefixe'    => 'CHAR(3)'
                    },
                    creator     => ['login'],
                    constructors => [['login']]
                }
                );

1;

