/*
 Copyright INRA/CNRS

 Emmanuel.Courcelle@toulouse.inra.fr
 Jerome.Gouzy@toulouse.inra.fr

 This software is a perl module whose purpose is to help you writing
 your own scripts

 This software is governed by the CeCILL license under French law and
 abiding by the rules of distribution of free software.  You can  use,
 modify and/ or redistribute the software under the terms of the CeCILL
 license as circulated by CEA, CNRS and INRIA at the following URL
 "http://www.cecill.info".

 As a counterpart to the access to the source code and  rights to copy,
 modify and redistribute granted by the license, users are provided only
 with a limited warranty  and the software's author,  the holder of the
 economic rights,  and the successive licensors  have only  limited
 liability.

 In this respect, the user's attention is drawn to the risks associated
 with loading,  using,  modifying and/or developing or reproducing the
 software by the user in light of its specific status of free software,
 that may mean  that it is complicated to manipulate,  and  that  also
 therefore means  that it is reserved for developers  and  experienced
 professionals having in-depth computer knowledge. Users are therefore
 encouraged to load and test the software's suitability as regards their
 requirements in conditions enabling the security of their systems and/or
 data to be ensured and,  more generally, to use and operate it in the
 same conditions as regards security.

 The fact that you are presently reading this means that you have had
 knowledge of the CeCILL license and that you accept its terms.

*/

/* 

=pod

=head1 NAME
 InitForm.js - Some functions useful to init forms and for elements

=cut
*/

INITFORM_SEPARATOR = ' ';

/*
=pod

=head2 InitForm

	Title       : InitForm
	Usage       : InitForm(form,o_param)
	Prerequiste :
	procedure   : Init the form from the values found in the o_params array
                  The array is scanned
                  For each key, a form element with the same name is searched and initialized
                  We try to detect the type of the element, in order to init it in an intelligent way
                  WARNING - 1/ If an element has several values (val1,val2,...), it is supposed that those values are joined,
                               using INITFORM_SEPARATOR as separator (see also ParamParser.pm)
                            2/ For a key called foo, if a key called hide_foo is provdied, and is value is 1, we look for an ancestor of the corresponding
                               element belonging to the class Removable, and we hide this ancestor (thus hiding all children)
                               The element, together its label etc. is then hidden, and its default value cannot be changed
                            3/ Checkboxes are handled as follows:
                                    -For each checkbox 'foo', you must provide a hidden field called 'ck_foo'
                                    -ALWAYS initialize the HIDDEN FIELD, with 1 or 0
                                    -This function initializes the checkbox FROM THE HIDDEN FIELD, Then keeps the box and the field in synchronization
    Globals     : None
	
=cut

*/

function InitForm(form,o_params)
{
	Element.extend(form);
	var h_params = $H(o_params);
	var ck_regex = /^ck_/;
	var param_keys  = h_params.keys();
	h_params.each(function(p) {
		 if (form[p.key] != undefined)
		 {
			 // f is a "field"
			 var f = $(form[p.key]);
			 var hide_key = 'hide_'+p.key;
			 var hide_flg = false;
			 var hide_val = "";
			 if (h_params.get(hide_key) != undefined)
			 {
				 //alert ('coucou '+hide_key);
				 hide_flg = true;
				 hide_val = h_params.get(hide_key);
			 }
			 
			 return Try.these(
			 
			 // We have an array of radios
			 function()
			 { 
				 if (f[0].type.toLowerCase()!='radio') {throw "error"};
				 // Keep track of the fields which should be removed
				 if (hide_flg)
				 {
					 var a_removed=$A();
					 a_removed=hide_val.split(INITFORM_SEPARATOR);
					 //alert(a_removed.inspect());
					 
					 // Remove some buttons
					 var btn_rmvd=0;
					 a_removed.each(function (r) {HideRemovableAncestor(GetRadio(f,r));btn_rmvd++;});
					 
					 // If ALL radio buttons are removed, we remove THE WHOLE set
					 // This works if the whole set is inside a Removable element (a fieldset for example)
					 if (btn_rmvd==f.length)
					 {
						 HideRemovableAncestor(f[0].up('.Removable'));
					 }
				 };
				 
				 // Check the correct button
				 SetRadioValue(f,p.value);
				 return true;
			 },
				 
			 // A select list, only one selection
			 function() 
			 {
				 if (f.type.toLowerCase()!='select-one') {throw 'error'};
				 f.setValue(p.value);
				 if (hide_flg){HideRemovableAncestor(f)};
				 return true;
			 },
			 
			 // A select list, multiple selections
			 function()
			 {
				 if (f.type.toLowerCase()!='select-multiple') {throw 'error'};
				 f.setValue(p.value.split(INITFORM_SEPARATOR));
				 if (hide_flg){HideRemovableAncestor(f)};
				 return true;
			 },
			 
			 // a checkbox
			 function()
			 {
				 if (f.type.toLowerCase()!='checkbox') {throw 'error'};
			
				 f.checked = p.value;
				 if (hide_flg){HideRemovableAncestor(f)};
				 return true;
			 },
				
			 // A hidden field called ck_XXX, it is linked to a checkbox
			 function()
			 {
				 // NOTE - Using webdevelopper, this will not work when you click "Display form details", as the hidden filed are no more marked as hidden
				 if (f.type.toLowerCase()!='hidden'||!ck_regex.test(p.key)){throw 'error'};

				 // look for a checkbox called XXX, throw an error if not found
				 var ck_key = p.key.substr(3,p.key.length-3); // remove the ck_
				 if (form[ck_key].type.toLowerCase()!='checkbox') {throw 'error'};
				 
				 // Set the hidden field's value and sync the box from the hidden field
				 f.value = p.value;
				 form[ck_key].checked = (p.value!='off' && p.value!=='0' && p.value!==0)?true:false;
				 
				 // Put an observer event to keep the synchro
				 // Synchronize the hidden from the checkbox, then fire an custom event (initform:change):
				 // we'll be able to observe that event, it is sent AFTER the synchronization
				 // NOTE - I hope this does not introduce memory leak if InitForm is called several times
				 Event.observe(form[ck_key],'click',function(evt){
					 form[p.key].value=(form[ck_key].checked)? 1 : 0;
					 form[p.key].fire('initform:change');
					 /*alert(form[ck_key].checked+" "+form[p.key].value);*/
				 });
				 
				 // If hide_ck_XXX==1, we hide the check box
				 if (hide_flg){HideRemovableAncestor(form[ck_key])};
			 },
			 
			 // Any other field (type text, hidden, ...)
			 function()
			 {
				 f.value=p.value;
				 if (hide_flg){HideRemovableAncestor(f)};
				 return true;
			 }) || false;
		 };
	});
};
				 
/*
=pod

=head2 HideRemovableAncestor

	Title       : HideRemovableAncestor
	Usage       : HideRemovableAncestor(some_element)
	procedure   : Hide the first ancestor belonging to the class Removable.
	              Everything is inside a try..catch, so that it is safe (but useless) removing something without any
				  Removable ancestor
    args        : elt the element to remove				  
	Globals     : None
	
=cut
*/

function HideRemovableAncestor(elt)
{
	try
	{
		elt.up('.Removable').hide();
	}
	catch(e){};
}


/*
=pod

=head2 DisableRadio,EnableRadio

	Title       : DisableRadio,EnableRadio
	Usage       : DisableRadio(form,some_name,'value1#value3')
	              EnableRadio(form,some_name,'value1#value3')
	Prerequiste : form[some_name] must be a 'radio buttons' element
	procedure   : Scan the buttons of the radio element passed by parameters
	              Each button whose name matches with one of the provided values is disabled/enabled
				  All other buttons are enabled/disabled
	Globals     : None

=cut
*/

function DisableRadio(form,name,value)
{
	var f = $(form);
	var r = form[name];
	Element.extend(r);
	if (r[0].type.toLowerCase()!='radio') {throw "ERROR: not a radio"};
	var a_values = value.split(INITFORM_SEPARATOR);
	for (var i=0; i<r.length;++i)
	{
		if (a_values.indexOf(r[i].value) != -1)
		{
			r[i].disable();
			r[i].checked  = false;
		}
		else
		{
			r[i].enable();
		}
	}
}

function EnableRadio(form,name,value)
{
	r = form[name];
	if (r[0].type.toLowerCase()!='radio') {throw "ERROR: not a radio"};
	var a_values = value.split(INITFORM_SEPARATOR);
	for (var i=0; i<r.length;++i)
	{
		if (a_values.indexOf(r[i].value) != -1)
		{
			r[i].disabled = false;
		}
		else
		{
			r[i].disabled = true;
			r[i].checked  = false;
		}
	}
}

/*
=pod

=head2 Function GetRadio

 Title        : GetRadio
 Prerequisite : At least 1 form element, type radio, must be part of the form
 Usage        : var value = GetRadio(value)
 Function     : Retrieve the input field whose value is passed by parameter, or undefined if it does not exist
 Arguments    : elt   The element to get the input field from (ex: form['some_radio_button']
                value The value to retrieve
 Return       : The input field (EXTENDED for prototype), or undefined
 Error        : throw an exception if form[name] is not an array of radio buttons
 Globals      : none

=cut
*/

function GetRadio(elt,value)
{
	// elt should be an array of input elements
	if (elt.length == undefined) {throw "the element is probably NOT a set of radio buttons";};
	
	for (var i=0; i<elt.length; ++i)
	{
		if (elt[i].value==value)
		{
			Element.extend(elt[i]);
			return elt[i];
		}
	}
	return undefined;
}

/*
=pod

=head2 Function GetRadioValue

 Title        : GetRadioValue
 Prerequisite : A Form element, type radio, must be part of the form
 Usage        : var value = GetRadioValue(elt)
 Function     : Retrieve the value of the checked element, or undefined if nothing is checked
 Arguments    : elt   The element to get (ex: form['some_radio_button']
 Return       : The value of the form element
 Error        : throw an exception if form[name] is not an array of radio buttons
 Globals      : none

=cut
*/

function GetRadioValue(elt)
{
	// elt should be an array of input elements
	if (elt.length == undefined) {throw "the element is probably NOT a set of radio buttons";};
	
	for (var i=0; i<elt.length; ++i)
	{
		if (elt[i].checked)
		{
			return elt[i].getValue();
		}
	}
	return undefined;
}

/*
=pod

=head2 Function SerializeRadio

 Title        : SerializeRadio
 Prerequisite : A Form element, type radio, must be part of the form
 Usage        : var serialized = SerializeRadio(elt)
 Function     : Retrieve the (name,value) of the checked element and return name=value
 Arguments    : elt   The element to serialize (ex: form['some_radio_button']
 Return       : The seiralized representation of the element
 Error        : throw an exception if form[name] is not an array of radio buttons
 Globals      : none

=cut
*/
function SerializeRadio(elt)
{
	var value= GetRadioValue(elt);
	var r0 = elt[0];
	var name = r0.name;
	return name + '=' + value;
}

/*
=pod

=head2 Function SetRadioValue

 Title        : SetRadioValue
 Prerequisite : A Form element, type radio, must be part of the form
 Usage        : var value = SetRadioValue(elt,value)
 Procedure    : check the element whose value is passed by parameter
 Arguments    : elt   The element to set (ex: form['some_radio_button']
				value The value to set
 Return       : nothing
 Error        : throw an exception if form[name] is not an array of radio buttons
                OR if there is no radio button with the correct value
 Globals      : none

=cut
*/

function SetRadioValue(elt, value)
{
	// form[name] should be an array
	if (elt.length == undefined) {throw "ERROR: the element is probably NOT a set of radio buttons";};
	var f = GetRadio(elt,value);
	if (f==undefined)
	{
		throw "ERROR: button with value " + value + " not found";
	}
	else
	{
		f.checked = true;
	}
}

/*
=pod

=head2 Function Serialize

 Title        : Serialize
 Usage        : var serialized = Serialize(elt)
 Function     : Try to call elt.serialize(), if it does not work try SerializeRadio
 Arguments    : elt   The element to serialize (ex: form['some_radio_button']
 Return       : The serialized representation of the element
 Error        : throw an exception if the element cannot be serialized at all
 Globals      : none

=cut
*/
function Serialize(elt)
{
	return Try.these(
	function() { return elt.serialize(); },
	function() { return SerializeRadio(elt); }
	);
}

/*
=pod

=head2 Function SetValue

 Title        : SetValue
 Usage        : SetValue(elt,value)
 Procedure    : Try to call set elt.value, if it does not work try SetRadioValue
 Arguments    : elt   The element to set (ex: form['something'])
 Error        : throw an exception if the value cannot be set at all (may be security reasons)
 Globals      : none

=cut
*/
function SetValue(elt,val)
{
	Try.these(
	function() { elt.value = val; },
	function() { SetRadioValue(elt,val);}
	);
}

/*
=pod

=head2 Function GetValue

 Title        : GetValue
 Usage        : var val = GetValue(elt)
 Function     : Try to call elt.getValue(), if it does not work try GetRadiovalue
 Arguments    : elt   The form element to get the value from
 Return       : the element's value
 Error        : throw an exception if the value cannot be get
 Globals      : none

=cut
*/
function GetValue(elt)
{
	return Try.these(
	function() { return elt.getValue(); },
	function() { return GetRadioValue(elt); }
	);
}
// TEMPORAIRE - PATCH MERDEUX
function Getvalue(elt){return GetValue(elt);};

/*
=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license under French law - www.cecill.info

=cut
*/

